# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: secret/secret.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from union.internal.secret import payload_pb2 as secret_dot_payload__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13secret/secret.proto\x12\x0f\x63loudidl.secret\x1a\x1cgoogle/api/annotations.proto\x1a\x14secret/payload.proto2\x86\x05\n\rSecretService\x12w\n\x0c\x43reateSecret\x12$.cloudidl.secret.CreateSecretRequest\x1a%.cloudidl.secret.CreateSecretResponse\"\x1a\x82\xd3\xe4\x93\x02\x14\"\x0f/secrets/api/v1:\x01*\x12\x86\x01\n\x0cUpdateSecret\x12$.cloudidl.secret.UpdateSecretRequest\x1a%.cloudidl.secret.UpdateSecretResponse\")\x82\xd3\xe4\x93\x02#\x1a\x1e/secrets/api/v1/name/{id.name}:\x01*\x12z\n\tGetSecret\x12!.cloudidl.secret.GetSecretRequest\x1a\".cloudidl.secret.GetSecretResponse\"&\x82\xd3\xe4\x93\x02 \x12\x1e/secrets/api/v1/name/{id.name}\x12\x83\x01\n\x0c\x44\x65leteSecret\x12$.cloudidl.secret.DeleteSecretRequest\x1a%.cloudidl.secret.DeleteSecretResponse\"&\x82\xd3\xe4\x93\x02 *\x1e/secrets/api/v1/name/{id.name}\x12q\n\x0bListSecrets\x12#.cloudidl.secret.ListSecretsRequest\x1a$.cloudidl.secret.ListSecretsResponse\"\x17\x82\xd3\xe4\x93\x02\x11\x12\x0f/secrets/api/v1B\xac\x01\n\x13\x63om.cloudidl.secretB\x0bSecretProtoH\x02P\x01Z)github.com/unionai/cloud/gen/pb-go/secret\xa2\x02\x03\x43SX\xaa\x02\x0f\x43loudidl.Secret\xca\x02\x0f\x43loudidl\\Secret\xe2\x02\x1b\x43loudidl\\Secret\\GPBMetadata\xea\x02\x10\x43loudidl::Secretb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'secret.secret_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023com.cloudidl.secretB\013SecretProtoH\002P\001Z)github.com/unionai/cloud/gen/pb-go/secret\242\002\003CSX\252\002\017Cloudidl.Secret\312\002\017Cloudidl\\Secret\342\002\033Cloudidl\\Secret\\GPBMetadata\352\002\020Cloudidl::Secret'
  _SECRETSERVICE.methods_by_name['CreateSecret']._options = None
  _SECRETSERVICE.methods_by_name['CreateSecret']._serialized_options = b'\202\323\344\223\002\024\"\017/secrets/api/v1:\001*'
  _SECRETSERVICE.methods_by_name['UpdateSecret']._options = None
  _SECRETSERVICE.methods_by_name['UpdateSecret']._serialized_options = b'\202\323\344\223\002#\032\036/secrets/api/v1/name/{id.name}:\001*'
  _SECRETSERVICE.methods_by_name['GetSecret']._options = None
  _SECRETSERVICE.methods_by_name['GetSecret']._serialized_options = b'\202\323\344\223\002 \022\036/secrets/api/v1/name/{id.name}'
  _SECRETSERVICE.methods_by_name['DeleteSecret']._options = None
  _SECRETSERVICE.methods_by_name['DeleteSecret']._serialized_options = b'\202\323\344\223\002 *\036/secrets/api/v1/name/{id.name}'
  _SECRETSERVICE.methods_by_name['ListSecrets']._options = None
  _SECRETSERVICE.methods_by_name['ListSecrets']._serialized_options = b'\202\323\344\223\002\021\022\017/secrets/api/v1'
  _globals['_SECRETSERVICE']._serialized_start=93
  _globals['_SECRETSERVICE']._serialized_end=739
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: identity/user_payload.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.common import identifier_pb2 as common_dot_identifier__pb2
from union.internal.common import identity_pb2 as common_dot_identity__pb2
from union.internal.common import list_pb2 as common_dot_list__pb2
from union.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1bidentity/user_payload.proto\x12\x11\x63loudidl.identity\x1a\x17\x63ommon/identifier.proto\x1a\x15\x63ommon/identity.proto\x1a\x11\x63ommon/list.proto\x1a\x17validate/validate.proto\"L\n\x11\x43reateUserRequest\x12\x37\n\x04spec\x18\x01 \x01(\x0b\x32\x19.cloudidl.common.UserSpecB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x04spec\"E\n\x12\x43reateUserResponse\x12/\n\x02id\x18\x01 \x01(\x0b\x32\x1f.cloudidl.common.UserIdentifierR\x02id\"K\n\x0eGetUserRequest\x12\x39\n\x02id\x18\x01 \x01(\x0b\x32\x1f.cloudidl.common.UserIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x02id\"<\n\x0fGetUserResponse\x12)\n\x04user\x18\x01 \x01(\x0b\x32\x15.cloudidl.common.UserR\x04user\"r\n\x11\x44\x65leteUserRequest\x12\x39\n\x02id\x18\x01 \x01(\x0b\x32\x1f.cloudidl.common.UserIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x02id\x12\"\n\x0corganization\x18\x02 \x01(\tR\x0corganization\"\x14\n\x12\x44\x65leteUserResponse\"\xa2\x02\n\x10ListUsersRequest\x12\"\n\x0corganization\x18\x01 \x01(\tR\x0corganization\x12<\n\x07role_id\x18\x02 \x01(\x0b\x32\x1f.cloudidl.common.RoleIdentifierB\x02\x18\x01R\x06roleId\x12\x36\n\x07request\x18\x03 \x01(\x0b\x32\x1c.cloudidl.common.ListRequestR\x07request\x12\x32\n\x15include_support_staff\x18\x04 \x01(\x08R\x13includeSupportStaff\x12@\n\x1c\x65xclude_identity_assignments\x18\x05 \x01(\x08R\x1a\x65xcludeIdentityAssignments\"V\n\x11ListUsersResponse\x12+\n\x05users\x18\x01 \x03(\x0b\x32\x15.cloudidl.common.UserR\x05users\x12\x14\n\x05token\x18\x02 \x01(\tR\x05token\"\x17\n\x15ListUsersCountRequest\".\n\x16ListUsersCountResponse\x12\x14\n\x05\x63ount\x18\x01 \x01(\x03R\x05\x63ount\"L\n\x1dGetUserAndGroupsForOrgRequest\x12+\n\x0corganization\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x0corganization\"M\n\x1eGetUserAndGroupsForOrgResponse\x12+\n\x05users\x18\x01 \x03(\x0b\x32\x15.cloudidl.common.UserR\x05usersB\xbd\x01\n\x15\x63om.cloudidl.identityB\x10UserPayloadProtoH\x02P\x01Z+github.com/unionai/cloud/gen/pb-go/identity\xa2\x02\x03\x43IX\xaa\x02\x11\x43loudidl.Identity\xca\x02\x11\x43loudidl\\Identity\xe2\x02\x1d\x43loudidl\\Identity\\GPBMetadata\xea\x02\x12\x43loudidl::Identityb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'identity.user_payload_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\025com.cloudidl.identityB\020UserPayloadProtoH\002P\001Z+github.com/unionai/cloud/gen/pb-go/identity\242\002\003CIX\252\002\021Cloudidl.Identity\312\002\021Cloudidl\\Identity\342\002\035Cloudidl\\Identity\\GPBMetadata\352\002\022Cloudidl::Identity'
  _CREATEUSERREQUEST.fields_by_name['spec']._options = None
  _CREATEUSERREQUEST.fields_by_name['spec']._serialized_options = b'\372B\005\212\001\002\020\001'
  _GETUSERREQUEST.fields_by_name['id']._options = None
  _GETUSERREQUEST.fields_by_name['id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _DELETEUSERREQUEST.fields_by_name['id']._options = None
  _DELETEUSERREQUEST.fields_by_name['id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _LISTUSERSREQUEST.fields_by_name['role_id']._options = None
  _LISTUSERSREQUEST.fields_by_name['role_id']._serialized_options = b'\030\001'
  _GETUSERANDGROUPSFORORGREQUEST.fields_by_name['organization']._options = None
  _GETUSERANDGROUPSFORORGREQUEST.fields_by_name['organization']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CREATEUSERREQUEST']._serialized_start=142
  _globals['_CREATEUSERREQUEST']._serialized_end=218
  _globals['_CREATEUSERRESPONSE']._serialized_start=220
  _globals['_CREATEUSERRESPONSE']._serialized_end=289
  _globals['_GETUSERREQUEST']._serialized_start=291
  _globals['_GETUSERREQUEST']._serialized_end=366
  _globals['_GETUSERRESPONSE']._serialized_start=368
  _globals['_GETUSERRESPONSE']._serialized_end=428
  _globals['_DELETEUSERREQUEST']._serialized_start=430
  _globals['_DELETEUSERREQUEST']._serialized_end=544
  _globals['_DELETEUSERRESPONSE']._serialized_start=546
  _globals['_DELETEUSERRESPONSE']._serialized_end=566
  _globals['_LISTUSERSREQUEST']._serialized_start=569
  _globals['_LISTUSERSREQUEST']._serialized_end=859
  _globals['_LISTUSERSRESPONSE']._serialized_start=861
  _globals['_LISTUSERSRESPONSE']._serialized_end=947
  _globals['_LISTUSERSCOUNTREQUEST']._serialized_start=949
  _globals['_LISTUSERSCOUNTREQUEST']._serialized_end=972
  _globals['_LISTUSERSCOUNTRESPONSE']._serialized_start=974
  _globals['_LISTUSERSCOUNTRESPONSE']._serialized_end=1020
  _globals['_GETUSERANDGROUPSFORORGREQUEST']._serialized_start=1022
  _globals['_GETUSERANDGROUPSFORORGREQUEST']._serialized_end=1098
  _globals['_GETUSERANDGROUPSFORORGRESPONSE']._serialized_start=1100
  _globals['_GETUSERANDGROUPSFORORGRESPONSE']._serialized_end=1177
# @@protoc_insertion_point(module_scope)



"""Module to track usage and errors."""

import logging
import os
import sys
from functools import partial
from os import getenv
from pathlib import Path
from typing import Any

import click
import flytekit
from click import Group
from flytekit.clis.sdk_in_container.utils import get_level_from_cli_verbosity, pretty_print_exception

if sys.version_info < (3, 10):
    from importlib_metadata import version
else:
    from importlib.metadata import version

import union

SENTRY_DSN = "https://8a7049a7fe42d7fe7b12e1b0d2171d21@o4507249423810560.ingest.us.sentry.io/4507249829281792"
SENTRY_DEBUG_DSN = "https://f9933bb58495729cffc5f6b97ea8502e@o4507249423810560.ingest.us.sentry.io/4507249428201472"
UNION_USAGE_STATS_ENABLED_ENV = "UNION_USAGE_STATS_ENABLED"
UNION_DEBUG_USAGE_STATS_ENV = "UNION_DEBUG_USAGE_STATS"


# Re-implementation of ErrorHandlingCommand.invoke to send error to sentry first
# Details: ErrorHandlingCommand.invoke captures the error and calls exit(1). This means
# we need to override `ErrorHandlingCommand.invoke` and capture the exceptions before
# `exit(1)`
# https://github.com/flyteorg/flytekit/blob/db2ff9e4e14bdcec2c96388af33f0a3357459fe0/flytekit/clis/sdk_in_container/utils.py#L123-L141
def custom_invoke(ctx: click.Context, self) -> Any:
    from flytekit.loggers import logger

    verbosity = ctx.params["verbose"]
    log_level = get_level_from_cli_verbosity(verbosity)
    logger.setLevel(log_level)
    try:
        return Group.invoke(self, ctx)
    except Exception as e:
        from union._exceptions import UnionRequireConfigException

        if not isinstance(e, UnionRequireConfigException):
            import sentry_sdk

            from union._config import _UNION_CONFIG

            if _UNION_CONFIG.config_obj is not None:
                endpoint = _UNION_CONFIG.config_obj.platform.endpoint
            else:
                endpoint = "unknown"

            sentry_sdk.set_tags(
                {
                    "config_endpoint": endpoint,
                    "org": _UNION_CONFIG.org,
                }
            )

            sentry_sdk.set_context(
                "configuration",
                {
                    "config_source": _UNION_CONFIG.config_source,
                    "config_endpoint": endpoint,
                    "org": _UNION_CONFIG.org,
                },
            )
            sentry_sdk.capture_exception(e)
        pretty_print_exception(e, verbosity)
        exit(1)


# Only show exceptions caused by union and flytekit.clients
PACKAGE_PATHS_TO_TRACK = union.__path__ + flytekit.clients.__path__


def _sentry_exit_callback(pending, timeout):
    """Silence 'Sentry is attempted to send... message"""


def _filter_exceptions(event, hint):
    """Filter out exceptions that are not from the union SDK."""
    try:
        source = event.get("exception") or event.get("threads")

        # Ignore events without a stack trace
        if source is None:
            return None

        last_frame = source["values"][0]["stacktrace"]["frames"][-1]
        exc_origin_path = last_frame["abs_path"]

        if not any(exc_origin_path.startswith(p) for p in PACKAGE_PATHS_TO_TRACK):
            return None
    except Exception:
        return None

    return event


def _configure_tracking_cli(main: click.Group) -> click.Group:
    """Configures the the union to catch exceptions."""
    # Be extra careful to not register tracking during execution
    if "FLYTE_INTERNAL_EXECUTION_ID" in os.environ:
        return main

    main.invoke = partial(custom_invoke, self=main)
    _init_tracking()
    return main


def _init_tracking():
    """Initializes tracking."""
    # Check if this is a development environment
    union_git_path = Path(union.__path__[0]).parent / ".git"
    in_dev_env = union_git_path.exists()

    # If UNIONAI_DEBUG_SENTRY_ENV != "0", then enable user stats for development.
    # If UNIONAI_DEBUG_SENTRY_ENV == "0", then disable user stats for development. (Default)
    enable_tracking_for_dev = getenv(UNION_DEBUG_USAGE_STATS_ENV, "0") != "0" and in_dev_env

    # If UNION_USAGE_STATS_ENABLED_ENV != "0", then enable user stats collection. (Default)
    # If UNION_USAGE_STATS_ENABLED_ENV == "0", then disable user stats collection.
    enable_usage_stats_for_prod = getenv(UNION_USAGE_STATS_ENABLED_ENV, "1") != "0" and not in_dev_env

    enable_usage_stats = enable_tracking_for_dev or enable_usage_stats_for_prod
    if not enable_usage_stats:
        return

    import sentry_sdk
    from sentry_sdk.integrations.atexit import AtexitIntegration
    from sentry_sdk.integrations.logging import LoggingIntegration

    sentry_logging = LoggingIntegration(
        level=logging.DEBUG,  # Capture debug and above as breadcrumbs
        event_level=logging.ERROR,  # Send errors as events
    )

    if enable_tracking_for_dev:
        # Use debug DSN for development, so we do not put these errors into the production tracker
        dsn = SENTRY_DEBUG_DSN
        release = "dev"
        environment = "development"
    else:
        dsn = SENTRY_DSN
        release = version("union")
        environment = "production"

    sentry_sdk.init(
        dsn=dsn,
        auto_enabling_integrations=False,
        before_send=_filter_exceptions,
        integrations=[AtexitIntegration(callback=_sentry_exit_callback), sentry_logging],
        enable_tracing=False,
        release=release,
        environment=environment,
        debug=enable_tracking_for_dev,
    )



from union.internal.common import identifier_pb2 as _identifier_pb2
from union.internal.common import list_pb2 as _list_pb2
from flyteidl.admin import execution_pb2 as _execution_pb2
from flyteidl.core import literals_pb2 as _literals_pb2
from union.internal.validate.validate import validate_pb2 as _validate_pb2
from union.internal.workspace import workspace_definition_pb2 as _workspace_definition_pb2
from union.internal.workspace import workspace_instance_pb2 as _workspace_instance_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class StartWorkspaceInstanceRequest(_message.Message):
    __slots__ = ["org", "project", "domain", "name", "workspace_definition_id", "spec", "inputs"]
    ORG_FIELD_NUMBER: _ClassVar[int]
    PROJECT_FIELD_NUMBER: _ClassVar[int]
    DOMAIN_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    WORKSPACE_DEFINITION_ID_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    INPUTS_FIELD_NUMBER: _ClassVar[int]
    org: str
    project: str
    domain: str
    name: str
    workspace_definition_id: _workspace_definition_pb2.WorkspaceDefinitionIdentifier
    spec: _execution_pb2.ExecutionSpec
    inputs: _literals_pb2.LiteralMap
    def __init__(self, org: _Optional[str] = ..., project: _Optional[str] = ..., domain: _Optional[str] = ..., name: _Optional[str] = ..., workspace_definition_id: _Optional[_Union[_workspace_definition_pb2.WorkspaceDefinitionIdentifier, _Mapping]] = ..., spec: _Optional[_Union[_execution_pb2.ExecutionSpec, _Mapping]] = ..., inputs: _Optional[_Union[_literals_pb2.LiteralMap, _Mapping]] = ...) -> None: ...

class StartWorkspaceInstanceResponse(_message.Message):
    __slots__ = ["workspace_instance"]
    WORKSPACE_INSTANCE_FIELD_NUMBER: _ClassVar[int]
    workspace_instance: _workspace_instance_pb2.WorkspaceInstance
    def __init__(self, workspace_instance: _Optional[_Union[_workspace_instance_pb2.WorkspaceInstance, _Mapping]] = ...) -> None: ...

class GetWorkspaceInstanceRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: _workspace_instance_pb2.WorkspaceInstanceIdentifier
    def __init__(self, id: _Optional[_Union[_workspace_instance_pb2.WorkspaceInstanceIdentifier, _Mapping]] = ...) -> None: ...

class GetWorkspaceInstanceResponse(_message.Message):
    __slots__ = ["workspace_instance"]
    WORKSPACE_INSTANCE_FIELD_NUMBER: _ClassVar[int]
    workspace_instance: _workspace_instance_pb2.WorkspaceInstance
    def __init__(self, workspace_instance: _Optional[_Union[_workspace_instance_pb2.WorkspaceInstance, _Mapping]] = ...) -> None: ...

class ListWorkspaceInstancesRequest(_message.Message):
    __slots__ = ["request", "org", "project", "workspace_definition_id", "principal"]
    REQUEST_FIELD_NUMBER: _ClassVar[int]
    ORG_FIELD_NUMBER: _ClassVar[int]
    PROJECT_FIELD_NUMBER: _ClassVar[int]
    WORKSPACE_DEFINITION_ID_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_FIELD_NUMBER: _ClassVar[int]
    request: _list_pb2.ListRequest
    org: str
    project: _identifier_pb2.ProjectIdentifier
    workspace_definition_id: _workspace_definition_pb2.WorkspaceDefinitionIdentifier
    principal: str
    def __init__(self, request: _Optional[_Union[_list_pb2.ListRequest, _Mapping]] = ..., org: _Optional[str] = ..., project: _Optional[_Union[_identifier_pb2.ProjectIdentifier, _Mapping]] = ..., workspace_definition_id: _Optional[_Union[_workspace_definition_pb2.WorkspaceDefinitionIdentifier, _Mapping]] = ..., principal: _Optional[str] = ...) -> None: ...

class ListWorkspaceInstancesResponse(_message.Message):
    __slots__ = ["workspace_instances", "token"]
    WORKSPACE_INSTANCES_FIELD_NUMBER: _ClassVar[int]
    TOKEN_FIELD_NUMBER: _ClassVar[int]
    workspace_instances: _containers.RepeatedCompositeFieldContainer[_workspace_instance_pb2.WorkspaceInstance]
    token: str
    def __init__(self, workspace_instances: _Optional[_Iterable[_Union[_workspace_instance_pb2.WorkspaceInstance, _Mapping]]] = ..., token: _Optional[str] = ...) -> None: ...

class WatchWorkspaceInstancesRequest(_message.Message):
    __slots__ = ["org", "project", "workspace_definition_id", "workspace_instance_id", "principal"]
    ORG_FIELD_NUMBER: _ClassVar[int]
    PROJECT_FIELD_NUMBER: _ClassVar[int]
    WORKSPACE_DEFINITION_ID_FIELD_NUMBER: _ClassVar[int]
    WORKSPACE_INSTANCE_ID_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_FIELD_NUMBER: _ClassVar[int]
    org: str
    project: _identifier_pb2.ProjectIdentifier
    workspace_definition_id: _workspace_definition_pb2.WorkspaceDefinitionIdentifier
    workspace_instance_id: _workspace_instance_pb2.WorkspaceInstanceIdentifier
    principal: str
    def __init__(self, org: _Optional[str] = ..., project: _Optional[_Union[_identifier_pb2.ProjectIdentifier, _Mapping]] = ..., workspace_definition_id: _Optional[_Union[_workspace_definition_pb2.WorkspaceDefinitionIdentifier, _Mapping]] = ..., workspace_instance_id: _Optional[_Union[_workspace_instance_pb2.WorkspaceInstanceIdentifier, _Mapping]] = ..., principal: _Optional[str] = ...) -> None: ...

class WatchWorkspaceInstancesResponse(_message.Message):
    __slots__ = ["start_event", "update_event", "stop_event", "stopping_event"]
    START_EVENT_FIELD_NUMBER: _ClassVar[int]
    UPDATE_EVENT_FIELD_NUMBER: _ClassVar[int]
    STOP_EVENT_FIELD_NUMBER: _ClassVar[int]
    STOPPING_EVENT_FIELD_NUMBER: _ClassVar[int]
    start_event: _workspace_instance_pb2.StartEvent
    update_event: _workspace_instance_pb2.UpdateEvent
    stop_event: _workspace_instance_pb2.StopEvent
    stopping_event: _workspace_instance_pb2.StoppingEvent
    def __init__(self, start_event: _Optional[_Union[_workspace_instance_pb2.StartEvent, _Mapping]] = ..., update_event: _Optional[_Union[_workspace_instance_pb2.UpdateEvent, _Mapping]] = ..., stop_event: _Optional[_Union[_workspace_instance_pb2.StopEvent, _Mapping]] = ..., stopping_event: _Optional[_Union[_workspace_instance_pb2.StoppingEvent, _Mapping]] = ...) -> None: ...

class StopWorkspaceInstanceRequest(_message.Message):
    __slots__ = ["id", "cause"]
    ID_FIELD_NUMBER: _ClassVar[int]
    CAUSE_FIELD_NUMBER: _ClassVar[int]
    id: _workspace_instance_pb2.WorkspaceInstanceIdentifier
    cause: str
    def __init__(self, id: _Optional[_Union[_workspace_instance_pb2.WorkspaceInstanceIdentifier, _Mapping]] = ..., cause: _Optional[str] = ...) -> None: ...

class StopWorkspaceInstanceResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...



import logging
from typing import Generator

import torch
import vllm.entrypoints.cli.main
from vllm.config import ModelConfig, VllmConfig
from vllm.distributed import get_tensor_model_parallel_rank
from vllm.model_executor.model_loader import get_model, register_model_loader
from vllm.model_executor.model_loader.default_loader import DefaultModelLoader
from vllm.model_executor.model_loader.sharded_state_loader import ShardedStateLoader
from vllm.model_executor.model_loader.utils import set_default_torch_dtype

from ..config import (
    LOCAL_MODEL_PATH,
    REMOTE_MODEL_PATH,
    STREAM_SAFETENSORS,
)
from ..loader import SafeTensorsStreamer, prefetch, prefix_exists

logger = logging.getLogger(__name__)


@register_model_loader("union-streaming")
class UnionModelLoader(DefaultModelLoader):
    def _get_weights_iterator(self, source) -> Generator[tuple[str, torch.Tensor], None, None]:
        # Try to load weights using the Union SafeTensorsLoader. Fallback to the default loader otherwise.
        try:
            streamer = SafeTensorsStreamer(REMOTE_MODEL_PATH, LOCAL_MODEL_PATH)
        except ValueError:
            return super()._get_weights_iterator(source)
        else:
            for name, tensor in streamer.get_tensors():
                yield source.prefix + name, tensor

    def download_model(self, model_config: ModelConfig) -> None:
        # Stream only
        pass

    def _load_sharded_model(self, vllm_config: VllmConfig, model_config: ModelConfig) -> torch.nn.Module:
        # Forked from: https://github.com/vllm-project/vllm/blob/99d01a5e3d5278284bad359ac8b87ee7a551afda/vllm/model_executor/model_loader/loader.py#L613
        # Sanity checks
        tensor_parallel_size = vllm_config.parallel_config.tensor_parallel_size
        rank = get_tensor_model_parallel_rank()
        if rank >= tensor_parallel_size:
            raise ValueError(f"Invalid rank {rank} for tensor parallel size {tensor_parallel_size}")
        with set_default_torch_dtype(vllm_config.model_config.dtype):
            with torch.device(vllm_config.device_config.device):
                model = get_model(vllm_config=vllm_config, model_config=model_config)
                for _, module in model.named_modules():
                    quant_method = getattr(module, "quant_method", None)
                    if quant_method is not None:
                        quant_method.process_weights_after_loading(module)
            state_dict = ShardedStateLoader._filter_subtensors(model.state_dict())
            streamer = SafeTensorsStreamer(
                REMOTE_MODEL_PATH,
                LOCAL_MODEL_PATH,
                rank=rank,
                tensor_parallel_size=tensor_parallel_size,
            )
            for name, tensor in streamer.get_tensors():
                # If loading with LoRA enabled, additional padding may
                # be added to certain parameters. We only load into a
                # narrowed view of the parameter data.
                param_data = state_dict[name].data
                param_shape = state_dict[name].shape
                for dim, size in enumerate(tensor.shape):
                    if size < param_shape[dim]:
                        param_data = param_data.narrow(dim, 0, size)
                if tensor.shape != param_shape:
                    logger.warning(
                        "loading tensor of shape %s into parameter '%s' of shape %s",
                        tensor.shape,
                        name,
                        param_shape,
                    )
                param_data.copy_(tensor)
                state_dict.pop(name)
            if state_dict:
                raise ValueError(f"Missing keys {tuple(state_dict)} in loaded state!")
        return model.eval()

    def load_model(
        self,
        vllm_config: VllmConfig,
        model_config: ModelConfig,
    ) -> torch.nn.Module:
        logger.info("Loading model with UnionModelLoader")
        if vllm_config.parallel_config.tensor_parallel_size > 1:
            return self._load_sharded_model(vllm_config)
        else:
            return super().load_model(vllm_config, model_config)


def main():
    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
    )

    # Prefetch the model
    if REMOTE_MODEL_PATH:
        if not prefix_exists(REMOTE_MODEL_PATH):
            raise FileNotFoundError(f"Model path not found: {REMOTE_MODEL_PATH}")

        prefetch(
            REMOTE_MODEL_PATH,
            LOCAL_MODEL_PATH,
            exclude_safetensors=STREAM_SAFETENSORS,
        )

    vllm.entrypoints.cli.main.main()



"""All imports to unionai in this file should be in the function definition.

This plugin is loaded by flytekit, so any imports to unionai can lead to circular imports.
"""

import hashlib
import json
import os
import re
import shutil
import sys
import tempfile
import warnings
from dataclasses import fields
from datetime import datetime, timedelta, timezone
from functools import lru_cache
from importlib.metadata import version
from pathlib import Path
from typing import ClassVar, List, Optional, Tuple

import click
from flytekit import Labels
from flytekit.configuration import SerializationSettings, TaskConfig
from flytekit.constants import CopyFileDetection
from flytekit.core.utils import str2bool
from flytekit.exceptions.user import FlyteEntityNotExistException
from flytekit.image_spec.image_spec import _F_IMG_ID, ImageBuildEngine, ImageSpec, ImageSpecBuilder
from flytekit.models import security
from flytekit.models import task as task_models
from flytekit.models.core import execution as core_execution_models
from flytekit.models.core.identifier import Identifier, ResourceType
from flytekit.models.filters import ValueIn
from flytekit.remote import FlyteRemote, FlyteTask, FlyteWorkflowExecution
from flytekit.remote.remote import MOST_RECENT_FIRST
from flytekit.tools.ignore import DockerIgnore, GitIgnore, IgnoreGroup, StandardIgnore
from flytekit.tools.script_mode import ls_files
from flytekit.tools.translator import Options

from union._utils import _sanitize_label

try:
    import tomllib
except ImportError:
    import tomli as tomllib


RECOVER_EXECUTION_PAGE_LIMIT = 1_000
UNIONAI_IMAGE_NAME_KEY = "unionai_image_name"

# Task suffix used to append to build-image task template.
# Used when we we are required to change the build-image task template
_BUILD_IMAGE_SUFFIX = "exec"
_BUILD_IMAGE_SUFFIX_HASH_ALGORITHM = "sha256"
_BUILD_IMAGE_SUFFIX_HASH_LENGTH = 8

_PACKAGE_NAME_RE = re.compile(r"^[\w-]+")
_IMAGE_BUILDER_PROJECT_DOMAIN = {"project": "system", "domain": "production"}
FLYTE_IMAGE_BUILD_IN_SYSTEM = str2bool(os.getenv("FLYTE_IMAGE_BUILD_IN_SYSTEM", "true"))


def _is_union(package: str) -> bool:
    """Return True if `package` is union or unionai."""
    m = _PACKAGE_NAME_RE.match(package)
    if not m:
        return False
    name = m.group()
    return name in ("union", "unionai")


def get_unionai_for_pypi():
    union_version = version("union")
    if "dev" in union_version:
        return "union"
    else:
        return f"union=={union_version}"


def _get_remote() -> FlyteRemote:
    from union.configuration import UnionAIPlugin
    from union.remote import UnionRemote

    cfg = TaskConfig.auto()
    if FLYTE_IMAGE_BUILD_IN_SYSTEM:
        project = _IMAGE_BUILDER_PROJECT_DOMAIN["project"]
        domain = _IMAGE_BUILDER_PROJECT_DOMAIN["domain"]
    else:
        from union.remote._remote import common_init

        project = common_init.project or cfg.project or _IMAGE_BUILDER_PROJECT_DOMAIN["project"]
        domain = common_init.domain or cfg.domain or _IMAGE_BUILDER_PROJECT_DOMAIN["domain"]
    remote = UnionRemote(
        config=UnionAIPlugin._get_config_for_remote(None, default_to_union_semantics=True),
        default_domain=domain,
        default_project=project,
    )
    return remote


@lru_cache
def _get_fully_qualified_image_name(remote, execution):
    return execution.outputs["fully_qualified_image"]


def _build_image_exec_version(components: List[str]) -> str:
    """
    Generate a consistent version, hash-based, for the build-image task.

    Args:
        components: List of strings to hash
        algorithm: Hash algorithm to use (md5, sha1, sha256, etc.)
        length: Optional length to truncate the hash to

    Returns:
        A hash string representing the array content
    """
    if not components:
        return hashlib.new(_BUILD_IMAGE_SUFFIX_HASH_ALGORITHM, b"").hexdigest()

    # Sort to ensure consistent order regardless of input order
    sorted_array = sorted(components)

    # Join with a delimiter that's unlikely to appear in the strings
    # Use unit separator (ASCII 31) as delimiter
    joined_string = "\u001f".join(sorted_array).encode("utf-8")

    hash_obj = hashlib.new(_BUILD_IMAGE_SUFFIX_HASH_ALGORITHM, joined_string)
    hash_result = hash_obj.hexdigest()

    if _BUILD_IMAGE_SUFFIX_HASH_LENGTH < len(hash_result):
        return hash_result[:_BUILD_IMAGE_SUFFIX_HASH_LENGTH]

    return hash_result


def _sanitize_image_name(image_name: str) -> str:
    """Sanitizes a Docker image name to be used as a Kubernetes label.

    Ensures the image name meets Kubernetes label requirements by sanitizing it.
    If the sanitized name exceeds 63 characters (Kubernetes label limit), it shortens
    the name by extracting just the image name and tag components.

    Args:
        image_name (str): The original Docker image name (may include registry, repository, name and tag)

    Returns:
        str: A sanitized image name that complies with Kubernetes label restrictions
    """

    # Shorten target_image if it exceeds 63 characters
    sanitized_target_image = _sanitize_label(image_name)
    if len(sanitized_target_image) > 63:
        # Parse the FQIN components
        parts = image_name.split("/")
        image_and_tag = parts[-1].split(":")
        image_name = image_and_tag[0]
        tag = image_and_tag[1] if len(image_and_tag) > 1 else "latest"

        # Just use image name and tag
        shortened_target = f"{image_name}:{tag}"
        sanitized_target_image = _sanitize_label(shortened_target)
    return sanitized_target_image


def _apply_task_template_changes(
    remote: FlyteRemote, task: FlyteTask, imagepull_secret_name: Optional[str]
) -> FlyteTask:
    """Creates a new task with image pull secret configuration.

    Takes an existing Flyte task and creates a new version of it with an image pull
    secret added to its security context. This allows the task to pull images from
    private registries during execution.

    Args:
        remote (FlyteRemote): The Flyte remote connection to use for registration
        task (FlyteTask): The original task to modify
        imagepull_secret_name (Optional[str]): The name of the image pull secret to add

    Returns:
        FlyteTask: A new task with the image pull secret configured, or the original
                   task if no imagepull_secret_name was provided
    """
    # Return the reference task if no imagepull secret is provided
    if not imagepull_secret_name:
        return task

    # Register the updated task with a new version
    name = f"{task.name}-{_BUILD_IMAGE_SUFFIX}"
    version = _build_image_exec_version([imagepull_secret_name, task.id.version])
    project = _IMAGE_BUILDER_PROJECT_DOMAIN["project"]
    domain = _IMAGE_BUILDER_PROJECT_DOMAIN["domain"]
    try:
        task = remote.fetch_task(name=name, version=version, project=project, domain=domain)
        click.secho(f"{task.name} already exists. Skipping registration.", fg="blue")
        return task
    except FlyteEntityNotExistException:
        click.secho(f"Task {name} does not exist. Registering a new version.", fg="blue")

        new_task_id = Identifier(
            resource_type=ResourceType.TASK,
            project=project,
            domain=domain,
            name=name,
            version=version,
        )

        template = task.template
        template._id = new_task_id
        template._security_context = security.SecurityContext(
            secrets=[
                security.Secret(key=imagepull_secret_name, mount_requirement=security.Secret.MountType.FILE),
            ]
        )
        task_spec = task_models.TaskSpec(template=template)
        new_task_id = remote.raw_register(task_spec, SerializationSettings(image_config=None), version)

        new_task = remote.fetch_task(name=new_task_id.name, version=new_task_id.version, project=project, domain=domain)

        return new_task


def _build(spec: Path, context: Optional[Path], target_image: str, imagepull_secret_name: Optional[str]) -> str:
    """Build image using UnionAI."""

    remote = _get_remote()
    start = datetime.now(timezone.utc)

    context_url = "" if context is None else remote.upload_file(context)[1]

    spec_url = remote.upload_file(spec)[1]
    task = remote.fetch_task(
        name="build-image",
        project=_IMAGE_BUILDER_PROJECT_DOMAIN["project"],
        domain=_IMAGE_BUILDER_PROJECT_DOMAIN["domain"],
    )
    entity = _apply_task_template_changes(remote, task, imagepull_secret_name)
    sanitized_name = _sanitize_image_name(target_image)
    execution = remote.execute(
        entity,
        inputs={"spec": spec_url, "context": context_url, "target_image": target_image},
        options=Options(
            labels=Labels(values={UNIONAI_IMAGE_NAME_KEY: sanitized_name}),
        ),
        overwrite_cache=True,
    )
    click.secho("👍 Build submitted!", bold=True, fg="yellow")

    console_url = remote.generate_console_url(execution)

    click.secho(
        "⏳ Waiting for build to finish at: " + click.style(console_url, fg="cyan"),
        bold=True,
    )
    execution = remote.wait(execution, poll_interval=timedelta(seconds=1))

    elapsed = str(datetime.now(timezone.utc) - start).split(".")[0]

    if execution.closure.phase == core_execution_models.WorkflowExecutionPhase.SUCCEEDED:
        click.secho(f"✅ Build completed in {elapsed}!", bold=True, fg="green")
    else:
        error_msg = execution.error.message
        raise click.ClickException(
            f"❌ Build failed in {elapsed} at {click.style(console_url, fg='cyan')} with error:\n\n{error_msg}"
        )

    return _get_fully_qualified_image_name(remote, execution)


def _copy_files_into_build_context(image_spec: ImageSpec, spec: dict, context_path: Path):
    if image_spec.source_copy_mode is not None and image_spec.source_copy_mode != CopyFileDetection.NO_COPY:
        if not image_spec.source_root:
            raise ValueError(f"Field source_root for {image_spec} must be set when copy is set")

        # Easter egg
        # Load in additional packages before installing pip/apt packages
        vendor_path = Path(image_spec.source_root) / ".vendor"
        if vendor_path.is_dir():
            spec["dist_dirpath"] = ".vendor"

        ignore = IgnoreGroup(image_spec.source_root, [GitIgnore, DockerIgnore, StandardIgnore])

        ls, _ = ls_files(
            str(image_spec.source_root), image_spec.source_copy_mode, deref_symlinks=False, ignore_group=ignore
        )

        for file_to_copy in ls:
            rel_path = os.path.relpath(file_to_copy, start=str(image_spec.source_root))
            Path(context_path / rel_path).parent.mkdir(parents=True, exist_ok=True)
            shutil.copy(
                file_to_copy,
                context_path / rel_path,
            )

    if image_spec.copy:
        for src in image_spec.copy:
            src_path = Path(src)

            if src_path.is_absolute() or ".." in src_path.parts:
                raise ValueError("Absolute paths or paths with '..' are not allowed in COPY command.")

            dst_path = context_path / src_path
            dst_path.parent.mkdir(parents=True, exist_ok=True)

            if src_path.is_dir():
                shutil.copytree(src_path, dst_path, dirs_exist_ok=True)
            else:
                shutil.copy(src_path, dst_path)


class UCImageSpecBuilder(ImageSpecBuilder):
    """ImageSpec builder for UnionAI."""

    _SUPPORTED_UNIVERSAL_PARAMETERS: ClassVar = {
        "name",
        "builder",
        "builder_options",
        "source_root",
        "env",
        "packages",
        "requirements",
        "apt_packages",
        "pip_index",
        "pip_extra_index_url",
        "pip_extra_args",
        "commands",
        "conda_packages",
        "conda_channels",
        "source_copy_mode",
        "registry",
        "copy",
    }

    _SUPPORTED_CUSTOM_BASE_IMAGE_PARAMETERS = _SUPPORTED_UNIVERSAL_PARAMETERS | {
        "base_image",
    }

    _SUPPORTED_UNION_BASE_IMAGE_PARAMETERS = _SUPPORTED_UNIVERSAL_PARAMETERS | {
        "python_version",
        "cuda",
        "cudnn",
        "platform",
    }

    def build_image(self, image_spec: ImageSpec):
        """Build image using UnionAI."""
        image_name = image_spec.image_name()

        with tempfile.TemporaryDirectory() as tmpdir:
            tmp_path = Path(tmpdir)
            spec_path, archive_path, imagepull_secret_name = self._validate_configuration(
                image_spec, tmp_path, image_name
            )
            return _build(spec_path, archive_path, image_name, imagepull_secret_name)

    def _print_submitting_new_build(self) -> bool:
        click.secho("🐳 Submitting a new build...", bold=True, fg="blue")
        return True

    def should_build(self, image_spec: ImageSpec) -> bool:
        """Check whether the image should be built."""
        image_name = image_spec.image_name()
        remote = _get_remote()

        _check_image_builder_enabled(remote)

        if image_spec.registry:
            click.secho(f"{image_name} configured with external registry.")
            return self._print_submitting_new_build()

        try:
            ImageBuildEngine._IMAGE_NAME_TO_REAL_NAME[image_name] = remote._get_image_fqin(image_name)
            click.secho(f"Image {image_name} found. Skip building.", fg="blue")
            return False
        except FlyteEntityNotExistException:
            click.secho(f"Image {image_name} was not found or has expired.", fg="blue")
            return self._print_submitting_new_build()

    def _validate_configuration(
        self, image_spec: ImageSpec, tmp_path: Path, image_name: str
    ) -> Tuple[Path, Optional[Path], Optional[str]]:
        """Validate and write configuration for builder."""
        field_names = [f.name for f in fields(ImageSpec) if not f.name.startswith("_")]

        if image_spec.base_image is None:  # Union base image
            unsupported_parameters = [
                name
                for name in field_names
                if getattr(image_spec, name) and name not in self._SUPPORTED_UNION_BASE_IMAGE_PARAMETERS
            ]
        else:  # Custom base image
            unsupported_parameters = [
                name
                for name in field_names
                if getattr(image_spec, name) and name not in self._SUPPORTED_CUSTOM_BASE_IMAGE_PARAMETERS
            ]

        if unsupported_parameters:
            msg = f"The following parameters are unsupported and ignored: {unsupported_parameters}"
            warnings.warn(msg, UserWarning)

        # Hardcoded for now since our base image only supports 3.11
        if image_spec.python_version is None:
            major = sys.version_info.major
            minor = sys.version_info.minor
        else:
            version_split = image_spec.python_version.split(".")
            if not (2 <= len(version_split) <= 3):
                raise ValueError("python_version must be in the form major.minor. For example: 3.12")
            major, minor = version_split[:2]
            try:
                major, minor = int(major), int(minor)
            except ValueError:
                raise ValueError("python_version must be in the form major.minor. For example: 3.12")

        if not (major == 3 and 9 <= minor <= 12):
            raise ValueError(f"Only python versions between 3.9 to 3.12 are supported, got {major}.{minor}")

        python_version = f"{major}.{minor}"

        spec = {"python_version": python_version}
        # Transform image spec into a spec we expect
        if image_spec.apt_packages:
            spec["apt_packages"] = image_spec.apt_packages
        if image_spec.commands:
            spec["commands"] = image_spec.commands
        if image_spec.cuda or image_spec.cudnn:
            spec["enable_gpu"] = True
        if image_spec.conda_packages:
            spec["conda_packages"] = image_spec.conda_packages
        if image_spec.conda_channels:
            spec["conda_channels"] = image_spec.conda_channels

        env = image_spec.env or {}
        env = {_F_IMG_ID: image_name, **env}
        if env:
            spec["env"] = env

        packages = []
        if image_spec.packages:
            packages.extend(image_spec.packages)

        spec["python_packages"] = packages

        pip_extra_index_urls = []
        if image_spec.pip_index:
            pip_extra_index_urls.append(image_spec.pip_index)

        if image_spec.pip_extra_index_url:
            pip_extra_index_urls.extend(image_spec.pip_extra_index_url)

        spec["pip_extra_index_urls"] = pip_extra_index_urls
        spec["pip_extra_args"] = image_spec.pip_extra_args

        context_path = tmp_path / "build.uc-image-builder"
        context_path.mkdir(exist_ok=True)

        _all_packages = []
        _all_packages.extend(packages)

        if image_spec.requirements:
            if not Path(image_spec.requirements).exists():
                raise ValueError(f"requirements file {image_spec.requirements} does not exist")
            name = Path(image_spec.requirements).name
            shutil.copy2(image_spec.requirements, context_path / name)
            spec["python_requirements_files"] = [name]
            if str(image_spec.requirements).endswith(".txt"):
                with (context_path / name).open() as f:
                    _all_packages.extend([line.strip() for line in f.readlines()])
            elif str(image_spec.requirements).endswith("poetry.lock"):
                pyproject = Path(image_spec.requirements).parent.joinpath("pyproject.toml")
                self._check_pyproject_for_unsupported_features(pyproject)
                self._check_image_spec_for_unsupported_features(image_spec)
                readme = self._get_package_readme(pyproject)
                if readme:
                    shutil.copy2(readme, context_path / readme.name)

                if pyproject.exists():
                    # almost never happens, but if the user has 'subdir/poetry.lock', note that this will copy the
                    # pyproject file directly into the context folder, removing the subdir.
                    shutil.copy2(pyproject, context_path / pyproject.name)
                else:
                    raise ValueError(f"must have project file if using lock file {image_spec.requirements}")

            elif str(image_spec.requirements).endswith("uv.lock"):
                self._prepare_for_uv_lock(image_spec, context_path)
                readme = self._get_package_readme(Path(image_spec.requirements).parent / "pyproject.toml")
                if readme:
                    shutil.copy2(readme, context_path / readme.name)

        # if union is not specified, then add it here.
        union_pypi_package = get_unionai_for_pypi()
        if not any(_is_union(p) for p in _all_packages) and union_pypi_package:
            packages.append(union_pypi_package)

        # Custom base image
        if image_spec.base_image:
            if not isinstance(image_spec.base_image, str):
                raise ValueError("Union image builder only supports string base images")
            spec["base_image"] = image_spec.base_image

        _copy_files_into_build_context(image_spec, spec, context_path)

        if any(context_path.iterdir()):
            archive_path = Path(shutil.make_archive(tmp_path / "context", "xztar", context_path))
        else:
            archive_path = None

        spec_path = tmp_path / "spec.json"
        with spec_path.open("w") as f:
            json.dump(spec, f)

        builder_options = image_spec.builder_options or {}

        return (spec_path, archive_path, builder_options.get("imagepull_secret_name"))

    @staticmethod
    def _prepare_for_uv_lock(image_spec: ImageSpec, context_path: Path):
        """Prepare image builder for uv.lock."""
        pyproject_path = Path(image_spec.requirements).parent / "pyproject.toml"

        if not pyproject_path.exists():
            raise ValueError("pyproject.toml must exist when using uv.lock")

        for line in pyproject_path.read_text().splitlines():
            if "tool.uv.index" in line:
                raise ValueError("External sources are not supported in pyproject.toml")

        if image_spec.packages is not None:
            raise ValueError(
                "When using uv.lock, setting packages is not supported. "
                "Please include your dependencies in the uv.lock file."
            )

        if image_spec.pip_index or image_spec.pip_extra_index_url:
            raise ValueError("External sources are not supported in image spec when using uv lock file")

        shutil.copy2(pyproject_path, context_path / pyproject_path.name)

    def _get_package_readme(self, pyproject_path: Path) -> Optional[Path]:
        with open(pyproject_path, "rb") as f:
            pyproject = tomllib.load(f)

        poetry_readme = pyproject.get("tool", {}).get("poetry", {}).get("readme", None)
        if poetry_readme:
            return Path(pyproject_path).parent / poetry_readme

        readme = pyproject.get("project", {}).get("readme", None)
        if readme:
            return Path(pyproject_path).parent / readme

        return None

    @staticmethod
    def _check_pyproject_for_unsupported_features(pyproject: Path):
        """
        Use of external sources is prohibited because caching might interfere with other builds, and because the
        image builder might not have network access to it.
        If there's enough request for this for non-pypi but public sources, we can build out smarter caching in all
        our builders.
        See https://python-poetry.org/docs/repositories/#package-sources
        """
        for line in pyproject.read_text().splitlines():
            if "tool.poetry.source" in line:
                raise ValueError("External sources are not supported in pyproject.toml")

    @staticmethod
    def _check_image_spec_for_unsupported_features(image_spec: ImageSpec):
        """
        Use of external sources is prohibited in poetry because it should be declared in the pyproject file instead
        (which in turn should raise an error because we don't want to support non-pypi sources).
        """
        if image_spec.pip_index:
            raise ValueError("External sources are not supported in image spec when using poetry lock file")
        if image_spec.pip_extra_index_url:
            raise ValueError("External sources are not supported in image spec when using poetry lock file")


def _check_image_builder_enabled(remote: FlyteRemote):
    """Checks if the Union image builder is enabled."""
    try:
        remote.fetch_task(
            name="build-image",
            project=_IMAGE_BUILDER_PROJECT_DOMAIN["project"],
            domain=_IMAGE_BUILDER_PROJECT_DOMAIN["domain"],
        )
    except FlyteEntityNotExistException as e:
        msg = "Union remote image builder is not enabled. Please contact Union support to enable it."
        raise click.ClickException(msg) from e


def _get_latest_image_build_execution(remote: FlyteRemote, image_name: str) -> Optional[FlyteWorkflowExecution]:
    """
    Get the latest image build execution for the given image name.
    Returns None if no execution is found or the execution is not in QUEUED, RUNNING, or SUCCEEDED phase.
    """
    # Replace ':' with '_' since flyte does not allow ':' in the label value
    target_image = _sanitize_label(image_name)
    executions, _ = remote.client.list_executions_paginated(
        limit=1,
        filters=[
            ValueIn("execution_tag.key", [UNIONAI_IMAGE_NAME_KEY]),
            ValueIn("execution_tag.value", [target_image]),
            ValueIn("phase", ["QUEUED", "RUNNING", "SUCCEEDED"]),
        ],
        sort_by=MOST_RECENT_FIRST,
        **_IMAGE_BUILDER_PROJECT_DOMAIN,
    )

    if len(executions) == 0:
        return None

    return remote.sync_execution(FlyteWorkflowExecution.promote_from_model(executions[0]))


def _register_union_image_builder(priority: int = 10):
    ImageBuildEngine.register("unionai", UCImageSpecBuilder(), priority)
    ImageBuildEngine.register("union", UCImageSpecBuilder(), priority)


def _is_output_not_found(e: Exception) -> bool:
    # Unfortunately, failure to find an execution output doesn't return a 404 but an exception that wraps the 404.
    # Here we check the exception string directly
    return "status code: 404" in str(e)


def get_image_name(image_url: str) -> str:
    """
    Extract the image name from a Docker image string.

    :param image_url: Docker image string (e.g., 'repository/name:tag' or 'name:tag')
    :return: The image name
    """
    pattern = r"^(?:[^/]+/)?([^:]+)(?::[^:]+)?$"
    match = re.match(pattern, image_url)
    if not match:
        raise ValueError(f"Invalid Docker image format: {image_url}")
    return match.group(1)


def is_union_image(image: str):
    return image.startswith("cr.union.ai/")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: workspace/workspace_instance.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.common import identity_pb2 as common_dot_identity__pb2
from flyteidl.core import identifier_pb2 as flyteidl_dot_core_dot_identifier__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from union.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2
from union.internal.workspace import workspace_definition_pb2 as workspace_dot_workspace__definition__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"workspace/workspace_instance.proto\x12\x12\x63loudidl.workspace\x1a\x15\x63ommon/identity.proto\x1a\x1e\x66lyteidl/core/identifier.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x17validate/validate.proto\x1a$workspace/workspace_definition.proto\"\xa4\x01\n\x1bWorkspaceInstanceIdentifier\x12\x10\n\x03org\x18\x01 \x01(\tR\x03org\x12!\n\x07project\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x07project\x12\x1f\n\x06\x64omain\x18\x03 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x06\x64omain\x12\x1b\n\x04name\x18\x04 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x04name\x12\x12\n\x04host\x18\x05 \x01(\tR\x04host\"\xfc\x02\n\x17WorkspaceInstanceStatus\x12X\n\x05phase\x18\x01 \x01(\x0e\x32\x42.cloudidl.workspace.WorkspaceInstanceStatus.WorkspaceInstancePhaseR\x05phase\"\x86\x02\n\x16WorkspaceInstancePhase\x12(\n$WORKSPACE_INSTANCE_PHASE_UNSPECIFIED\x10\x00\x12$\n WORKSPACE_INSTANCE_PHASE_PENDING\x10\x01\x12$\n WORKSPACE_INSTANCE_PHASE_RUNNING\x10\x02\x12(\n$WORKSPACE_INSTANCE_PHASE_TERMINATING\x10\x03\x12\'\n#WORKSPACE_INSTANCE_PHASE_TERMINATED\x10\x04\x12#\n\x1fWORKSPACE_INSTANCE_PHASE_FAILED\x10\x05\"\xf0\x03\n\x15WorkspaceInstanceSpec\x12\x43\n\x06status\x18\x01 \x01(\x0b\x32+.cloudidl.workspace.WorkspaceInstanceStatusR\x06status\x12\x35\n\x08identity\x18\x02 \x01(\x0b\x32\x19.cloudidl.common.IdentityR\x08identity\x12\x39\n\ncreated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12s\n\x17workspace_definition_id\x18\x05 \x01(\x0b\x32\x31.cloudidl.workspace.WorkspaceDefinitionIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x15workspaceDefinitionId\x12^\n\x15workflow_execution_id\x18\x06 \x01(\x0b\x32*.flyteidl.core.WorkflowExecutionIdentifierR\x13workflowExecutionId\x12\x10\n\x03uri\x18\x07 \x01(\tR\x03uri\"\xa7\x01\n\x11WorkspaceInstance\x12I\n\x02id\x18\x01 \x01(\x0b\x32/.cloudidl.workspace.WorkspaceInstanceIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x02id\x12G\n\x04spec\x18\x02 \x01(\x0b\x32).cloudidl.workspace.WorkspaceInstanceSpecB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x04spec\"Y\n\x12WorkspaceInstances\x12\x43\n\tinstances\x18\x01 \x03(\x0b\x32%.cloudidl.workspace.WorkspaceInstanceR\tinstances\"Q\n\nStartEvent\x12\x43\n\tworkspace\x18\x01 \x01(\x0b\x32%.cloudidl.workspace.WorkspaceInstanceR\tworkspace\"\xad\x01\n\x0bUpdateEvent\x12R\n\x11updated_workspace\x18\x01 \x01(\x0b\x32%.cloudidl.workspace.WorkspaceInstanceR\x10updatedWorkspace\x12J\n\rold_workspace\x18\x02 \x01(\x0b\x32%.cloudidl.workspace.WorkspaceInstanceR\x0coldWorkspace\"L\n\tStopEvent\x12?\n\x02id\x18\x01 \x01(\x0b\x32/.cloudidl.workspace.WorkspaceInstanceIdentifierR\x02id\"P\n\rStoppingEvent\x12?\n\x02id\x18\x01 \x01(\x0b\x32/.cloudidl.workspace.WorkspaceInstanceIdentifierR\x02idB\xc9\x01\n\x16\x63om.cloudidl.workspaceB\x16WorkspaceInstanceProtoH\x02P\x01Z,github.com/unionai/cloud/gen/pb-go/workspace\xa2\x02\x03\x43WX\xaa\x02\x12\x43loudidl.Workspace\xca\x02\x12\x43loudidl\\Workspace\xe2\x02\x1e\x43loudidl\\Workspace\\GPBMetadata\xea\x02\x13\x43loudidl::Workspaceb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'workspace.workspace_instance_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.cloudidl.workspaceB\026WorkspaceInstanceProtoH\002P\001Z,github.com/unionai/cloud/gen/pb-go/workspace\242\002\003CWX\252\002\022Cloudidl.Workspace\312\002\022Cloudidl\\Workspace\342\002\036Cloudidl\\Workspace\\GPBMetadata\352\002\023Cloudidl::Workspace'
  _WORKSPACEINSTANCEIDENTIFIER.fields_by_name['project']._options = None
  _WORKSPACEINSTANCEIDENTIFIER.fields_by_name['project']._serialized_options = b'\372B\004r\002\020\001'
  _WORKSPACEINSTANCEIDENTIFIER.fields_by_name['domain']._options = None
  _WORKSPACEINSTANCEIDENTIFIER.fields_by_name['domain']._serialized_options = b'\372B\004r\002\020\001'
  _WORKSPACEINSTANCEIDENTIFIER.fields_by_name['name']._options = None
  _WORKSPACEINSTANCEIDENTIFIER.fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _WORKSPACEINSTANCESPEC.fields_by_name['workspace_definition_id']._options = None
  _WORKSPACEINSTANCESPEC.fields_by_name['workspace_definition_id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _WORKSPACEINSTANCE.fields_by_name['id']._options = None
  _WORKSPACEINSTANCE.fields_by_name['id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _WORKSPACEINSTANCE.fields_by_name['spec']._options = None
  _WORKSPACEINSTANCE.fields_by_name['spec']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_WORKSPACEINSTANCEIDENTIFIER']._serialized_start=210
  _globals['_WORKSPACEINSTANCEIDENTIFIER']._serialized_end=374
  _globals['_WORKSPACEINSTANCESTATUS']._serialized_start=377
  _globals['_WORKSPACEINSTANCESTATUS']._serialized_end=757
  _globals['_WORKSPACEINSTANCESTATUS_WORKSPACEINSTANCEPHASE']._serialized_start=495
  _globals['_WORKSPACEINSTANCESTATUS_WORKSPACEINSTANCEPHASE']._serialized_end=757
  _globals['_WORKSPACEINSTANCESPEC']._serialized_start=760
  _globals['_WORKSPACEINSTANCESPEC']._serialized_end=1256
  _globals['_WORKSPACEINSTANCE']._serialized_start=1259
  _globals['_WORKSPACEINSTANCE']._serialized_end=1426
  _globals['_WORKSPACEINSTANCES']._serialized_start=1428
  _globals['_WORKSPACEINSTANCES']._serialized_end=1517
  _globals['_STARTEVENT']._serialized_start=1519
  _globals['_STARTEVENT']._serialized_end=1600
  _globals['_UPDATEEVENT']._serialized_start=1603
  _globals['_UPDATEEVENT']._serialized_end=1776
  _globals['_STOPEVENT']._serialized_start=1778
  _globals['_STOPEVENT']._serialized_end=1854
  _globals['_STOPPINGEVENT']._serialized_start=1856
  _globals['_STOPPINGEVENT']._serialized_end=1936
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: imagebuilder/definition.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dimagebuilder/definition.proto\x12\x15\x63loudidl.imagebuilder\x1a\x17validate/validate.proto\".\n\x0fImageIdentifier\x12\x1b\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x04name\"S\n\x05Image\x12\x36\n\x02id\x18\x01 \x01(\x0b\x32&.cloudidl.imagebuilder.ImageIdentifierR\x02id\x12\x12\n\x04\x66qin\x18\x02 \x01(\tR\x04\x66qinB\xd4\x01\n\x19\x63om.cloudidl.imagebuilderB\x0f\x44\x65\x66initionProtoH\x02P\x01Z/github.com/unionai/cloud/gen/pb-go/imagebuilder\xa2\x02\x03\x43IX\xaa\x02\x15\x43loudidl.Imagebuilder\xca\x02\x15\x43loudidl\\Imagebuilder\xe2\x02!Cloudidl\\Imagebuilder\\GPBMetadata\xea\x02\x16\x43loudidl::Imagebuilderb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'imagebuilder.definition_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\031com.cloudidl.imagebuilderB\017DefinitionProtoH\002P\001Z/github.com/unionai/cloud/gen/pb-go/imagebuilder\242\002\003CIX\252\002\025Cloudidl.Imagebuilder\312\002\025Cloudidl\\Imagebuilder\342\002!Cloudidl\\Imagebuilder\\GPBMetadata\352\002\026Cloudidl::Imagebuilder'
  _IMAGEIDENTIFIER.fields_by_name['name']._options = None
  _IMAGEIDENTIFIER.fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_IMAGEIDENTIFIER']._serialized_start=81
  _globals['_IMAGEIDENTIFIER']._serialized_end=127
  _globals['_IMAGE']._serialized_start=129
  _globals['_IMAGE']._serialized_end=212
# @@protoc_insertion_point(module_scope)



import logging
from pathlib import Path
from typing import Optional

import rich.markup
import rich_click as click
from rich.console import Console

from union.app import App
from union.cli._option import MutuallyExclusiveArgument, MutuallyExclusiveOption
from union.internal.secret.definition_pb2 import SecretIdentifier, SecretSpec
from union.internal.secret.payload_pb2 import UpdateSecretRequest
from union.remote import UnionRemote
from union.workspace._vscode import WorkspaceConfig, build_workspace_image

logger = logging.getLogger("union.cli._update")


@click.group(name="update")
def update():
    """Update a resource."""


@update.command()
@click.argument(
    "name",
    required=False,
    cls=MutuallyExclusiveArgument,
    mutually_exclusive=["name_option"],
    error_msg="Please pass --name once: `union update secret --name NAME`",
)
@click.option(
    "--name",
    "name_option",
    help="Secret name",
    cls=MutuallyExclusiveOption,
    mutually_exclusive=["name"],
    error_msg="Please pass --name once: `union update secret --name NAME`",
)
@click.option(
    "--value",
    help="Secret value",
    prompt="Enter secret value",
    hide_input=True,
    cls=MutuallyExclusiveOption,
    mutually_exclusive=["value_file"],
)
@click.option(
    "-f",
    "--value-file",
    help="Path to file containing the secret",
    type=click.Path(exists=True, file_okay=True, dir_okay=False, readable=True, allow_dash=True),
    cls=MutuallyExclusiveOption,
    mutually_exclusive=["value"],
)
@click.option("--project", help="Project name")
@click.option("--domain", help="Domain name")
def secret(
    name: Optional[str],
    name_option: Optional[str],
    value: str,
    value_file: str,
    project: Optional[str],
    domain: Optional[str],
):
    """Update secret with NAME."""
    name = name or name_option

    if value_file:
        with open(value_file, "rb") as f:
            secret_spec = SecretSpec(binary_value=f.read())
    else:
        secret_spec = SecretSpec(string_value=value)

    remote = UnionRemote(default_domain=domain, default_project=project)
    stub = remote.secret_client

    request = UpdateSecretRequest(
        id=SecretIdentifier(name=name, domain=domain, project=project),
        secret_spec=secret_spec,
    )

    try:
        stub.UpdateSecret(request)
        click.echo(f"Updated secret with name: {name}")
    except Exception as e:
        raise click.ClickException(f"Unable to update secret with name: {name}\n{e}") from e


def update_application(app: App, project: str, domain: str):
    from union.configuration import UnionAIPlugin

    remote = UnionAIPlugin.get_remote(config=None, project=project, domain=domain)
    remote._app_remote.update(app)


@update.command()
@click.pass_context
@click.argument("config_file", type=click.Path(path_type=Path))
def workspace(ctx: click.Context, config_file: Path):
    """Update a workspace."""
    config_content = config_file.read_text()
    workspace_config = WorkspaceConfig.from_yaml(config_content)

    remote = UnionRemote(default_domain=workspace_config.domain, default_project=workspace_config.project)
    logger.debug(f"Workspace config: {workspace_config}")
    logger.debug(f"Remote: {remote.__dict__}")
    if not isinstance(workspace_config.container_image, str):
        _image_build_task = build_workspace_image(remote, workspace_config)
        # override container image with the built image
        workspace_config.container_image = _image_build_task.template.container.image
    _update_workspace(remote, workspace_config)


def _update_workspace(remote: UnionRemote, ws_config: WorkspaceConfig):
    from union._async import run_sync
    from union.internal.workspace.workspace_definition_payload_pb2 import CreateWorkspaceDefinitionResponse
    from union.workspace._vscode_remote import WorkspaceRemote

    ws_remote = WorkspaceRemote(
        default_project=ws_config.project,
        default_domain=ws_config.domain,
        union_remote=remote,
    )

    console = Console()
    try:
        ws_def: CreateWorkspaceDefinitionResponse = run_sync(ws_remote.update_workspace_definition, ws_config)
    except Exception as e:
        raise click.ClickException(f"Error updating workspace definition: {rich.markup.escape(str(e))}") from e

    console.print(f"Updated: {ws_def}")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: authorizer/authorizer.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.authorizer import payload_pb2 as authorizer_dot_payload__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x61uthorizer/authorizer.proto\x12\x13\x63loudidl.authorizer\x1a\x18\x61uthorizer/payload.proto\x1a\x1cgoogle/api/annotations.proto2\x8d \n\x11\x41uthorizerService\x12\x83\x01\n\tAuthorize\x12%.cloudidl.authorizer.AuthorizeRequest\x1a&.cloudidl.authorizer.AuthorizeResponse\"\'\x82\xd3\xe4\x93\x02!\"\x1c/authorizer/api/v1/authorize:\x01*\x12\x93\x01\n\x0e\x43reateIdentity\x12*.cloudidl.authorizer.CreateIdentityRequest\x1a+.cloudidl.authorizer.CreateIdentityResponse\"(\x82\xd3\xe4\x93\x02\"\"\x1d/authorizer/api/v1/identities:\x01*\x12\x90\x01\n\x0e\x44\x65leteIdentity\x12*.cloudidl.authorizer.DeleteIdentityRequest\x1a+.cloudidl.authorizer.DeleteIdentityResponse\"%\x82\xd3\xe4\x93\x02\x1f*\x1d/authorizer/api/v1/identities\x12\x9f\x01\n\x0e\x41ssignIdentity\x12*.cloudidl.authorizer.AssignIdentityRequest\x1a+.cloudidl.authorizer.AssignIdentityResponse\"4\x82\xd3\xe4\x93\x02.\x1a)/authorizer/api/v1/identities/assignments:\x01*\x12\xa5\x01\n\x10UnassignIdentity\x12,.cloudidl.authorizer.UnassignIdentityRequest\x1a-.cloudidl.authorizer.UnassignIdentityResponse\"4\x82\xd3\xe4\x93\x02.*)/authorizer/api/v1/identities/assignments:\x01*\x12\xc8\x02\n\x16GetIdentityAssignments\x12\x31.cloudidl.authorizer.GetIdentityAssignmentRequest\x1a\x32.cloudidl.authorizer.GetIdentityAssignmentResponse\"\xc6\x01\x82\xd3\xe4\x93\x02\xbf\x01\x12)/authorizer/api/v1/identities/assignmentsZO\x12M/authorizer/api/v1/applications/assignments/{identity.application_id.subject}ZA\x12?/authorizer/api/v1/users/assignments/{identity.user_id.subject}\x12\xbc\x01\n\x17ListIdentityAssignments\x12\x33.cloudidl.authorizer.ListIdentityAssignmentsRequest\x1a\x34.cloudidl.authorizer.ListIdentityAssignmentsResponse\"6\x82\xd3\xe4\x93\x02\x30\x12./authorizer/api/v1/identities/assignments/list\x12\xb1\x01\n\x13GetPolicyAssignment\x12/.cloudidl.authorizer.GetPolicyAssignmentRequest\x1a\x30.cloudidl.authorizer.GetPolicyAssignmentResponse\"7\x82\xd3\xe4\x93\x02\x31\x12//authorizer/api/v1/policy_assignments/{id.name}\x12\x92\x01\n\x0e\x43reateResource\x12*.cloudidl.authorizer.CreateResourceRequest\x1a+.cloudidl.authorizer.CreateResourceResponse\"\'\x82\xd3\xe4\x93\x02!\"\x1c/authorizer/api/v1/resources:\x01*\x12\x92\x01\n\x0e\x44\x65leteResource\x12*.cloudidl.authorizer.DeleteResourceRequest\x1a+.cloudidl.authorizer.DeleteResourceResponse\"\'\x82\xd3\xe4\x93\x02!*\x1c/authorizer/api/v1/resources:\x01*\x12\xa2\x01\n\x12\x43reateOrganization\x12..cloudidl.authorizer.CreateOrganizationRequest\x1a/.cloudidl.authorizer.CreateOrganizationResponse\"+\x82\xd3\xe4\x93\x02%\" /authorizer/api/v1/organizations:\x01*\x12\xa5\x01\n\x0fGetOrganization\x12+.cloudidl.authorizer.GetOrganizationRequest\x1a,.cloudidl.authorizer.GetOrganizationResponse\"7\x82\xd3\xe4\x93\x02\x31\x12//authorizer/api/v1/organizations/{organization}\x12\xb1\x01\n\x12\x44\x65leteOrganization\x12..cloudidl.authorizer.DeleteOrganizationRequest\x1a/.cloudidl.authorizer.DeleteOrganizationResponse\":\x82\xd3\xe4\x93\x02\x34*//authorizer/api/v1/organizations/{organization}:\x01*\x12\x8b\x01\n\x0c\x43reatePolicy\x12(.cloudidl.authorizer.CreatePolicyRequest\x1a).cloudidl.authorizer.CreatePolicyResponse\"&\x82\xd3\xe4\x93\x02 \"\x1b/authorizer/api/v1/policies:\x01*\x12\x89\x01\n\tGetPolicy\x12%.cloudidl.authorizer.GetPolicyRequest\x1a&.cloudidl.authorizer.GetPolicyResponse\"-\x82\xd3\xe4\x93\x02\'\x12%/authorizer/api/v1/policies/{id.name}\x12\x92\x01\n\x0c\x44\x65letePolicy\x12(.cloudidl.authorizer.DeletePolicyRequest\x1a).cloudidl.authorizer.DeletePolicyResponse\"-\x82\xd3\xe4\x93\x02\'*%/authorizer/api/v1/policies/{id.name}\x12\x88\x01\n\x0cListPolicies\x12(.cloudidl.authorizer.ListPoliciesRequest\x1a).cloudidl.authorizer.ListPoliciesResponse\"#\x82\xd3\xe4\x93\x02\x1d\x12\x1b/authorizer/api/v1/policies\x12\xba\x01\n\x13\x43reatePolicyBinding\x12/.cloudidl.authorizer.CreatePolicyBindingRequest\x1a\x30.cloudidl.authorizer.CreatePolicyBindingResponse\"@\x82\xd3\xe4\x93\x02:\"5/authorizer/api/v1/policies/{policy_id.name}/bindings:\x01*\x12\xba\x01\n\x13\x44\x65letePolicyBinding\x12/.cloudidl.authorizer.DeletePolicyBindingRequest\x1a\x30.cloudidl.authorizer.DeletePolicyBindingResponse\"@\x82\xd3\xe4\x93\x02:*5/authorizer/api/v1/policies/{policy_id.name}/bindings:\x01*\x12\x82\x01\n\nCreateRole\x12&.cloudidl.authorizer.CreateRoleRequest\x1a\'.cloudidl.authorizer.CreateRoleResponse\"#\x82\xd3\xe4\x93\x02\x1d\"\x18/authorizer/api/v1/roles:\x01*\x12\x80\x01\n\x07GetRole\x12#.cloudidl.authorizer.GetRoleRequest\x1a$.cloudidl.authorizer.GetRoleResponse\"*\x82\xd3\xe4\x93\x02$\x12\"/authorizer/api/v1/roles/{id.name}\x12\x89\x01\n\nDeleteRole\x12&.cloudidl.authorizer.DeleteRoleRequest\x1a\'.cloudidl.authorizer.DeleteRoleResponse\"*\x82\xd3\xe4\x93\x02$*\"/authorizer/api/v1/roles/{id.name}\x12|\n\tListRoles\x12%.cloudidl.authorizer.ListRolesRequest\x1a&.cloudidl.authorizer.ListRolesResponse\" \x82\xd3\xe4\x93\x02\x1a\x12\x18/authorizer/api/v1/roles\x12\xa6\x01\n\x10\x43reateRoleAction\x12,.cloudidl.authorizer.CreateRoleActionRequest\x1a-.cloudidl.authorizer.CreateRoleActionResponse\"5\x82\xd3\xe4\x93\x02/\"*/authorizer/api/v1/roles/{id.name}/actions:\x01*\x12\xa6\x01\n\x10\x44\x65leteRoleAction\x12,.cloudidl.authorizer.DeleteRoleActionRequest\x1a-.cloudidl.authorizer.DeleteRoleActionResponse\"5\x82\xd3\xe4\x93\x02/**/authorizer/api/v1/roles/{id.name}/actions:\x01*B\xc8\x01\n\x17\x63om.cloudidl.authorizerB\x0f\x41uthorizerProtoH\x02P\x01Z-github.com/unionai/cloud/gen/pb-go/authorizer\xa2\x02\x03\x43\x41X\xaa\x02\x13\x43loudidl.Authorizer\xca\x02\x13\x43loudidl\\Authorizer\xe2\x02\x1f\x43loudidl\\Authorizer\\GPBMetadata\xea\x02\x14\x43loudidl::Authorizerb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'authorizer.authorizer_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\027com.cloudidl.authorizerB\017AuthorizerProtoH\002P\001Z-github.com/unionai/cloud/gen/pb-go/authorizer\242\002\003CAX\252\002\023Cloudidl.Authorizer\312\002\023Cloudidl\\Authorizer\342\002\037Cloudidl\\Authorizer\\GPBMetadata\352\002\024Cloudidl::Authorizer'
  _AUTHORIZERSERVICE.methods_by_name['Authorize']._options = None
  _AUTHORIZERSERVICE.methods_by_name['Authorize']._serialized_options = b'\202\323\344\223\002!\"\034/authorizer/api/v1/authorize:\001*'
  _AUTHORIZERSERVICE.methods_by_name['CreateIdentity']._options = None
  _AUTHORIZERSERVICE.methods_by_name['CreateIdentity']._serialized_options = b'\202\323\344\223\002\"\"\035/authorizer/api/v1/identities:\001*'
  _AUTHORIZERSERVICE.methods_by_name['DeleteIdentity']._options = None
  _AUTHORIZERSERVICE.methods_by_name['DeleteIdentity']._serialized_options = b'\202\323\344\223\002\037*\035/authorizer/api/v1/identities'
  _AUTHORIZERSERVICE.methods_by_name['AssignIdentity']._options = None
  _AUTHORIZERSERVICE.methods_by_name['AssignIdentity']._serialized_options = b'\202\323\344\223\002.\032)/authorizer/api/v1/identities/assignments:\001*'
  _AUTHORIZERSERVICE.methods_by_name['UnassignIdentity']._options = None
  _AUTHORIZERSERVICE.methods_by_name['UnassignIdentity']._serialized_options = b'\202\323\344\223\002.*)/authorizer/api/v1/identities/assignments:\001*'
  _AUTHORIZERSERVICE.methods_by_name['GetIdentityAssignments']._options = None
  _AUTHORIZERSERVICE.methods_by_name['GetIdentityAssignments']._serialized_options = b'\202\323\344\223\002\277\001\022)/authorizer/api/v1/identities/assignmentsZO\022M/authorizer/api/v1/applications/assignments/{identity.application_id.subject}ZA\022?/authorizer/api/v1/users/assignments/{identity.user_id.subject}'
  _AUTHORIZERSERVICE.methods_by_name['ListIdentityAssignments']._options = None
  _AUTHORIZERSERVICE.methods_by_name['ListIdentityAssignments']._serialized_options = b'\202\323\344\223\0020\022./authorizer/api/v1/identities/assignments/list'
  _AUTHORIZERSERVICE.methods_by_name['GetPolicyAssignment']._options = None
  _AUTHORIZERSERVICE.methods_by_name['GetPolicyAssignment']._serialized_options = b'\202\323\344\223\0021\022//authorizer/api/v1/policy_assignments/{id.name}'
  _AUTHORIZERSERVICE.methods_by_name['CreateResource']._options = None
  _AUTHORIZERSERVICE.methods_by_name['CreateResource']._serialized_options = b'\202\323\344\223\002!\"\034/authorizer/api/v1/resources:\001*'
  _AUTHORIZERSERVICE.methods_by_name['DeleteResource']._options = None
  _AUTHORIZERSERVICE.methods_by_name['DeleteResource']._serialized_options = b'\202\323\344\223\002!*\034/authorizer/api/v1/resources:\001*'
  _AUTHORIZERSERVICE.methods_by_name['CreateOrganization']._options = None
  _AUTHORIZERSERVICE.methods_by_name['CreateOrganization']._serialized_options = b'\202\323\344\223\002%\" /authorizer/api/v1/organizations:\001*'
  _AUTHORIZERSERVICE.methods_by_name['GetOrganization']._options = None
  _AUTHORIZERSERVICE.methods_by_name['GetOrganization']._serialized_options = b'\202\323\344\223\0021\022//authorizer/api/v1/organizations/{organization}'
  _AUTHORIZERSERVICE.methods_by_name['DeleteOrganization']._options = None
  _AUTHORIZERSERVICE.methods_by_name['DeleteOrganization']._serialized_options = b'\202\323\344\223\0024*//authorizer/api/v1/organizations/{organization}:\001*'
  _AUTHORIZERSERVICE.methods_by_name['CreatePolicy']._options = None
  _AUTHORIZERSERVICE.methods_by_name['CreatePolicy']._serialized_options = b'\202\323\344\223\002 \"\033/authorizer/api/v1/policies:\001*'
  _AUTHORIZERSERVICE.methods_by_name['GetPolicy']._options = None
  _AUTHORIZERSERVICE.methods_by_name['GetPolicy']._serialized_options = b'\202\323\344\223\002\'\022%/authorizer/api/v1/policies/{id.name}'
  _AUTHORIZERSERVICE.methods_by_name['DeletePolicy']._options = None
  _AUTHORIZERSERVICE.methods_by_name['DeletePolicy']._serialized_options = b'\202\323\344\223\002\'*%/authorizer/api/v1/policies/{id.name}'
  _AUTHORIZERSERVICE.methods_by_name['ListPolicies']._options = None
  _AUTHORIZERSERVICE.methods_by_name['ListPolicies']._serialized_options = b'\202\323\344\223\002\035\022\033/authorizer/api/v1/policies'
  _AUTHORIZERSERVICE.methods_by_name['CreatePolicyBinding']._options = None
  _AUTHORIZERSERVICE.methods_by_name['CreatePolicyBinding']._serialized_options = b'\202\323\344\223\002:\"5/authorizer/api/v1/policies/{policy_id.name}/bindings:\001*'
  _AUTHORIZERSERVICE.methods_by_name['DeletePolicyBinding']._options = None
  _AUTHORIZERSERVICE.methods_by_name['DeletePolicyBinding']._serialized_options = b'\202\323\344\223\002:*5/authorizer/api/v1/policies/{policy_id.name}/bindings:\001*'
  _AUTHORIZERSERVICE.methods_by_name['CreateRole']._options = None
  _AUTHORIZERSERVICE.methods_by_name['CreateRole']._serialized_options = b'\202\323\344\223\002\035\"\030/authorizer/api/v1/roles:\001*'
  _AUTHORIZERSERVICE.methods_by_name['GetRole']._options = None
  _AUTHORIZERSERVICE.methods_by_name['GetRole']._serialized_options = b'\202\323\344\223\002$\022\"/authorizer/api/v1/roles/{id.name}'
  _AUTHORIZERSERVICE.methods_by_name['DeleteRole']._options = None
  _AUTHORIZERSERVICE.methods_by_name['DeleteRole']._serialized_options = b'\202\323\344\223\002$*\"/authorizer/api/v1/roles/{id.name}'
  _AUTHORIZERSERVICE.methods_by_name['ListRoles']._options = None
  _AUTHORIZERSERVICE.methods_by_name['ListRoles']._serialized_options = b'\202\323\344\223\002\032\022\030/authorizer/api/v1/roles'
  _AUTHORIZERSERVICE.methods_by_name['CreateRoleAction']._options = None
  _AUTHORIZERSERVICE.methods_by_name['CreateRoleAction']._serialized_options = b'\202\323\344\223\002/\"*/authorizer/api/v1/roles/{id.name}/actions:\001*'
  _AUTHORIZERSERVICE.methods_by_name['DeleteRoleAction']._options = None
  _AUTHORIZERSERVICE.methods_by_name['DeleteRoleAction']._serialized_options = b'\202\323\344\223\002/**/authorizer/api/v1/roles/{id.name}/actions:\001*'
  _globals['_AUTHORIZERSERVICE']._serialized_start=109
  _globals['_AUTHORIZERSERVICE']._serialized_end=4218
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: workspace/workspace_definition_payload.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.common import list_pb2 as common_dot_list__pb2
from flyteidl.admin import task_pb2 as flyteidl_dot_admin_dot_task__pb2
from union.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2
from union.internal.workspace import workspace_definition_pb2 as workspace_dot_workspace__definition__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,workspace/workspace_definition_payload.proto\x12\x12\x63loudidl.workspace\x1a\x11\x63ommon/list.proto\x1a\x19\x66lyteidl/admin/task.proto\x1a\x17validate/validate.proto\x1a$workspace/workspace_definition.proto\"\xf3\x02\n CreateWorkspaceDefinitionRequest\x12K\n\x02id\x18\x01 \x01(\x0b\x32\x31.cloudidl.workspace.WorkspaceDefinitionIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x02id\x12\x30\n\x04spec\x18\x02 \x01(\x0b\x32\x18.flyteidl.admin.TaskSpecB\x02\x18\x01R\x04spec\x12/\n\x11short_description\x18\x03 \x01(\tB\x02\x18\x01R\x10shortDescription\x12R\n\x0eworkspace_spec\x18\x04 \x01(\x0b\x32+.cloudidl.workspace.WorkspaceDefinitionSpecR\rworkspaceSpec\x12K\n\"create_workspace_definition_entity\x18\x05 \x01(\x08R\x1f\x63reateWorkspaceDefinitionEntity\"\x7f\n!CreateWorkspaceDefinitionResponse\x12Z\n\x14workspace_definition\x18\x01 \x01(\x0b\x32\'.cloudidl.workspace.WorkspaceDefinitionR\x13workspaceDefinition\"l\n\x1dGetWorkspaceDefinitionRequest\x12K\n\x02id\x18\x01 \x01(\x0b\x32\x31.cloudidl.workspace.WorkspaceDefinitionIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x02id\"|\n\x1eGetWorkspaceDefinitionResponse\x12Z\n\x14workspace_definition\x18\x01 \x01(\x0b\x32\'.cloudidl.workspace.WorkspaceDefinitionR\x13workspaceDefinition\"\xf2\x02\n UpdateWorkspaceDefinitionRequest\x12K\n\x02id\x18\x01 \x01(\x0b\x32\x31.cloudidl.workspace.WorkspaceDefinitionIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x02id\x12\x30\n\x04spec\x18\x02 \x01(\x0b\x32\x18.flyteidl.admin.TaskSpecB\x02\x18\x01R\x04spec\x12\x46\n\x05state\x18\x03 \x01(\x0e\x32,.cloudidl.workspace.WorkspaceDefinitionStateB\x02\x18\x01R\x05state\x12/\n\x11short_description\x18\x04 \x01(\tB\x02\x18\x01R\x10shortDescription\x12R\n\x0eworkspace_spec\x18\x05 \x01(\x0b\x32+.cloudidl.workspace.WorkspaceDefinitionSpecR\rworkspaceSpec:\x02\x18\x01\"\x7f\n!UpdateWorkspaceDefinitionResponse\x12Z\n\x14workspace_definition\x18\x01 \x01(\x0b\x32\'.cloudidl.workspace.WorkspaceDefinitionR\x13workspaceDefinition\"\xc3\x01\n\x1fListWorkspaceDefinitionsRequest\x12\x10\n\x03org\x18\x01 \x01(\tR\x03org\x12!\n\x07project\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x07project\x12\x1f\n\x06\x64omain\x18\x03 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x06\x64omain\x12\x12\n\x04name\x18\x05 \x01(\tR\x04name\x12\x36\n\x07request\x18\x04 \x01(\x0b\x32\x1c.cloudidl.common.ListRequestR\x07request\"\x96\x01\n ListWorkspaceDefinitionsResponse\x12\\\n\x15workspace_definitions\x18\x01 \x03(\x0b\x32\'.cloudidl.workspace.WorkspaceDefinitionR\x14workspaceDefinitions\x12\x14\n\x05token\x18\x02 \x01(\tR\x05token\"\xb6\x01\n&ListWorkspaceDefinitionEntitiesRequest\x12\x10\n\x03org\x18\x01 \x01(\tR\x03org\x12!\n\x07project\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x07project\x12\x1f\n\x06\x64omain\x18\x03 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x06\x64omain\x12\x36\n\x07request\x18\x04 \x01(\x0b\x32\x1c.cloudidl.common.ListRequestR\x07request\"\xb2\x01\n\'ListWorkspaceDefinitionEntitiesResponse\x12q\n\x1dworkspace_definition_entities\x18\x01 \x03(\x0b\x32-.cloudidl.workspace.WorkspaceDefinitionEntityR\x1bworkspaceDefinitionEntities\x12\x14\n\x05token\x18\x02 \x01(\tR\x05token\"y\n&UpdateWorkspaceDefinitionEntityRequest\x12O\n\x06\x65ntity\x18\x01 \x01(\x0b\x32-.cloudidl.workspace.WorkspaceDefinitionEntityB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x06\x65ntity\")\n\'UpdateWorkspaceDefinitionEntityResponseB\xd2\x01\n\x16\x63om.cloudidl.workspaceB\x1fWorkspaceDefinitionPayloadProtoH\x02P\x01Z,github.com/unionai/cloud/gen/pb-go/workspace\xa2\x02\x03\x43WX\xaa\x02\x12\x43loudidl.Workspace\xca\x02\x12\x43loudidl\\Workspace\xe2\x02\x1e\x43loudidl\\Workspace\\GPBMetadata\xea\x02\x13\x43loudidl::Workspaceb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'workspace.workspace_definition_payload_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.cloudidl.workspaceB\037WorkspaceDefinitionPayloadProtoH\002P\001Z,github.com/unionai/cloud/gen/pb-go/workspace\242\002\003CWX\252\002\022Cloudidl.Workspace\312\002\022Cloudidl\\Workspace\342\002\036Cloudidl\\Workspace\\GPBMetadata\352\002\023Cloudidl::Workspace'
  _CREATEWORKSPACEDEFINITIONREQUEST.fields_by_name['id']._options = None
  _CREATEWORKSPACEDEFINITIONREQUEST.fields_by_name['id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _CREATEWORKSPACEDEFINITIONREQUEST.fields_by_name['spec']._options = None
  _CREATEWORKSPACEDEFINITIONREQUEST.fields_by_name['spec']._serialized_options = b'\030\001'
  _CREATEWORKSPACEDEFINITIONREQUEST.fields_by_name['short_description']._options = None
  _CREATEWORKSPACEDEFINITIONREQUEST.fields_by_name['short_description']._serialized_options = b'\030\001'
  _GETWORKSPACEDEFINITIONREQUEST.fields_by_name['id']._options = None
  _GETWORKSPACEDEFINITIONREQUEST.fields_by_name['id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _UPDATEWORKSPACEDEFINITIONREQUEST.fields_by_name['id']._options = None
  _UPDATEWORKSPACEDEFINITIONREQUEST.fields_by_name['id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _UPDATEWORKSPACEDEFINITIONREQUEST.fields_by_name['spec']._options = None
  _UPDATEWORKSPACEDEFINITIONREQUEST.fields_by_name['spec']._serialized_options = b'\030\001'
  _UPDATEWORKSPACEDEFINITIONREQUEST.fields_by_name['state']._options = None
  _UPDATEWORKSPACEDEFINITIONREQUEST.fields_by_name['state']._serialized_options = b'\030\001'
  _UPDATEWORKSPACEDEFINITIONREQUEST.fields_by_name['short_description']._options = None
  _UPDATEWORKSPACEDEFINITIONREQUEST.fields_by_name['short_description']._serialized_options = b'\030\001'
  _UPDATEWORKSPACEDEFINITIONREQUEST._options = None
  _UPDATEWORKSPACEDEFINITIONREQUEST._serialized_options = b'\030\001'
  _LISTWORKSPACEDEFINITIONSREQUEST.fields_by_name['project']._options = None
  _LISTWORKSPACEDEFINITIONSREQUEST.fields_by_name['project']._serialized_options = b'\372B\004r\002\020\001'
  _LISTWORKSPACEDEFINITIONSREQUEST.fields_by_name['domain']._options = None
  _LISTWORKSPACEDEFINITIONSREQUEST.fields_by_name['domain']._serialized_options = b'\372B\004r\002\020\001'
  _LISTWORKSPACEDEFINITIONENTITIESREQUEST.fields_by_name['project']._options = None
  _LISTWORKSPACEDEFINITIONENTITIESREQUEST.fields_by_name['project']._serialized_options = b'\372B\004r\002\020\001'
  _LISTWORKSPACEDEFINITIONENTITIESREQUEST.fields_by_name['domain']._options = None
  _LISTWORKSPACEDEFINITIONENTITIESREQUEST.fields_by_name['domain']._serialized_options = b'\372B\004r\002\020\001'
  _UPDATEWORKSPACEDEFINITIONENTITYREQUEST.fields_by_name['entity']._options = None
  _UPDATEWORKSPACEDEFINITIONENTITYREQUEST.fields_by_name['entity']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_CREATEWORKSPACEDEFINITIONREQUEST']._serialized_start=178
  _globals['_CREATEWORKSPACEDEFINITIONREQUEST']._serialized_end=549
  _globals['_CREATEWORKSPACEDEFINITIONRESPONSE']._serialized_start=551
  _globals['_CREATEWORKSPACEDEFINITIONRESPONSE']._serialized_end=678
  _globals['_GETWORKSPACEDEFINITIONREQUEST']._serialized_start=680
  _globals['_GETWORKSPACEDEFINITIONREQUEST']._serialized_end=788
  _globals['_GETWORKSPACEDEFINITIONRESPONSE']._serialized_start=790
  _globals['_GETWORKSPACEDEFINITIONRESPONSE']._serialized_end=914
  _globals['_UPDATEWORKSPACEDEFINITIONREQUEST']._serialized_start=917
  _globals['_UPDATEWORKSPACEDEFINITIONREQUEST']._serialized_end=1287
  _globals['_UPDATEWORKSPACEDEFINITIONRESPONSE']._serialized_start=1289
  _globals['_UPDATEWORKSPACEDEFINITIONRESPONSE']._serialized_end=1416
  _globals['_LISTWORKSPACEDEFINITIONSREQUEST']._serialized_start=1419
  _globals['_LISTWORKSPACEDEFINITIONSREQUEST']._serialized_end=1614
  _globals['_LISTWORKSPACEDEFINITIONSRESPONSE']._serialized_start=1617
  _globals['_LISTWORKSPACEDEFINITIONSRESPONSE']._serialized_end=1767
  _globals['_LISTWORKSPACEDEFINITIONENTITIESREQUEST']._serialized_start=1770
  _globals['_LISTWORKSPACEDEFINITIONENTITIESREQUEST']._serialized_end=1952
  _globals['_LISTWORKSPACEDEFINITIONENTITIESRESPONSE']._serialized_start=1955
  _globals['_LISTWORKSPACEDEFINITIONENTITIESRESPONSE']._serialized_end=2133
  _globals['_UPDATEWORKSPACEDEFINITIONENTITYREQUEST']._serialized_start=2135
  _globals['_UPDATEWORKSPACEDEFINITIONENTITYREQUEST']._serialized_end=2256
  _globals['_UPDATEWORKSPACEDEFINITIONENTITYRESPONSE']._serialized_start=2258
  _globals['_UPDATEWORKSPACEDEFINITIONENTITYRESPONSE']._serialized_end=2299
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: app/replica_definition.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.app import app_definition_pb2 as app_dot_app__definition__pb2
from union.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1c\x61pp/replica_definition.proto\x12\x0c\x63loudidl.app\x1a\x18\x61pp/app_definition.proto\x1a\x17validate/validate.proto\"k\n\x11ReplicaIdentifier\x12\x39\n\x06\x61pp_id\x18\x01 \x01(\x0b\x32\x18.cloudidl.app.IdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x05\x61ppId\x12\x1b\n\x04name\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x04name\"d\n\x0bReplicaMeta\x12\x39\n\x02id\x18\x01 \x01(\x0b\x32\x1f.cloudidl.app.ReplicaIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x02id\x12\x1a\n\x08revision\x18\x02 \x01(\x04R\x08revision\":\n\x0bReplicaList\x12+\n\x05items\x18\x01 \x03(\x0b\x32\x15.cloudidl.app.ReplicaR\x05items\"\x7f\n\x07Replica\x12?\n\x08metadata\x18\x01 \x01(\x0b\x32\x19.cloudidl.app.ReplicaMetaB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x08metadata\x12\x33\n\x06status\x18\x02 \x01(\x0b\x32\x1b.cloudidl.app.ReplicaStatusR\x06status\"T\n\rReplicaStatus\x12+\n\x11\x64\x65ployment_status\x18\x01 \x01(\tR\x10\x64\x65ploymentStatus\x12\x16\n\x06reason\x18\x02 \x01(\tR\x06reasonB\xa5\x01\n\x10\x63om.cloudidl.appB\x16ReplicaDefinitionProtoH\x02P\x01Z&github.com/unionai/cloud/gen/pb-go/app\xa2\x02\x03\x43\x41X\xaa\x02\x0c\x43loudidl.App\xca\x02\x0c\x43loudidl\\App\xe2\x02\x18\x43loudidl\\App\\GPBMetadata\xea\x02\rCloudidl::Appb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'app.replica_definition_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\020com.cloudidl.appB\026ReplicaDefinitionProtoH\002P\001Z&github.com/unionai/cloud/gen/pb-go/app\242\002\003CAX\252\002\014Cloudidl.App\312\002\014Cloudidl\\App\342\002\030Cloudidl\\App\\GPBMetadata\352\002\rCloudidl::App'
  _REPLICAIDENTIFIER.fields_by_name['app_id']._options = None
  _REPLICAIDENTIFIER.fields_by_name['app_id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _REPLICAIDENTIFIER.fields_by_name['name']._options = None
  _REPLICAIDENTIFIER.fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _REPLICAMETA.fields_by_name['id']._options = None
  _REPLICAMETA.fields_by_name['id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _REPLICA.fields_by_name['metadata']._options = None
  _REPLICA.fields_by_name['metadata']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_REPLICAIDENTIFIER']._serialized_start=97
  _globals['_REPLICAIDENTIFIER']._serialized_end=204
  _globals['_REPLICAMETA']._serialized_start=206
  _globals['_REPLICAMETA']._serialized_end=306
  _globals['_REPLICALIST']._serialized_start=308
  _globals['_REPLICALIST']._serialized_end=366
  _globals['_REPLICA']._serialized_start=368
  _globals['_REPLICA']._serialized_end=495
  _globals['_REPLICASTATUS']._serialized_start=497
  _globals['_REPLICASTATUS']._serialized_end=581
# @@protoc_insertion_point(module_scope)



from union.internal.common import identifier_pb2 as _identifier_pb2
from union.internal.common import identity_pb2 as _identity_pb2
from union.internal.common import list_pb2 as _list_pb2
from union.internal.validate.validate import validate_pb2 as _validate_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class CreateUserRequest(_message.Message):
    __slots__ = ["spec"]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    spec: _identity_pb2.UserSpec
    def __init__(self, spec: _Optional[_Union[_identity_pb2.UserSpec, _Mapping]] = ...) -> None: ...

class CreateUserResponse(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: _identifier_pb2.UserIdentifier
    def __init__(self, id: _Optional[_Union[_identifier_pb2.UserIdentifier, _Mapping]] = ...) -> None: ...

class GetUserRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: _identifier_pb2.UserIdentifier
    def __init__(self, id: _Optional[_Union[_identifier_pb2.UserIdentifier, _Mapping]] = ...) -> None: ...

class GetUserResponse(_message.Message):
    __slots__ = ["user"]
    USER_FIELD_NUMBER: _ClassVar[int]
    user: _identity_pb2.User
    def __init__(self, user: _Optional[_Union[_identity_pb2.User, _Mapping]] = ...) -> None: ...

class DeleteUserRequest(_message.Message):
    __slots__ = ["id", "organization"]
    ID_FIELD_NUMBER: _ClassVar[int]
    ORGANIZATION_FIELD_NUMBER: _ClassVar[int]
    id: _identifier_pb2.UserIdentifier
    organization: str
    def __init__(self, id: _Optional[_Union[_identifier_pb2.UserIdentifier, _Mapping]] = ..., organization: _Optional[str] = ...) -> None: ...

class DeleteUserResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class ListUsersRequest(_message.Message):
    __slots__ = ["organization", "role_id", "request", "include_support_staff", "exclude_identity_assignments"]
    ORGANIZATION_FIELD_NUMBER: _ClassVar[int]
    ROLE_ID_FIELD_NUMBER: _ClassVar[int]
    REQUEST_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_SUPPORT_STAFF_FIELD_NUMBER: _ClassVar[int]
    EXCLUDE_IDENTITY_ASSIGNMENTS_FIELD_NUMBER: _ClassVar[int]
    organization: str
    role_id: _identifier_pb2.RoleIdentifier
    request: _list_pb2.ListRequest
    include_support_staff: bool
    exclude_identity_assignments: bool
    def __init__(self, organization: _Optional[str] = ..., role_id: _Optional[_Union[_identifier_pb2.RoleIdentifier, _Mapping]] = ..., request: _Optional[_Union[_list_pb2.ListRequest, _Mapping]] = ..., include_support_staff: bool = ..., exclude_identity_assignments: bool = ...) -> None: ...

class ListUsersResponse(_message.Message):
    __slots__ = ["users", "token"]
    USERS_FIELD_NUMBER: _ClassVar[int]
    TOKEN_FIELD_NUMBER: _ClassVar[int]
    users: _containers.RepeatedCompositeFieldContainer[_identity_pb2.User]
    token: str
    def __init__(self, users: _Optional[_Iterable[_Union[_identity_pb2.User, _Mapping]]] = ..., token: _Optional[str] = ...) -> None: ...

class ListUsersCountRequest(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class ListUsersCountResponse(_message.Message):
    __slots__ = ["count"]
    COUNT_FIELD_NUMBER: _ClassVar[int]
    count: int
    def __init__(self, count: _Optional[int] = ...) -> None: ...

class GetUserAndGroupsForOrgRequest(_message.Message):
    __slots__ = ["organization"]
    ORGANIZATION_FIELD_NUMBER: _ClassVar[int]
    organization: str
    def __init__(self, organization: _Optional[str] = ...) -> None: ...

class GetUserAndGroupsForOrgResponse(_message.Message):
    __slots__ = ["users"]
    USERS_FIELD_NUMBER: _ClassVar[int]
    users: _containers.RepeatedCompositeFieldContainer[_identity_pb2.User]
    def __init__(self, users: _Optional[_Iterable[_Union[_identity_pb2.User, _Mapping]]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: imagebuilder/payload.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.common import identifier_pb2 as common_dot_identifier__pb2
from union.internal.imagebuilder import definition_pb2 as imagebuilder_dot_definition__pb2
from union.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1aimagebuilder/payload.proto\x12\x15\x63loudidl.imagebuilder\x1a\x17\x63ommon/identifier.proto\x1a\x1dimagebuilder/definition.proto\x1a\x17validate/validate.proto\"\xba\x01\n\x0fGetImageRequest\x12@\n\x02id\x18\x01 \x01(\x0b\x32&.cloudidl.imagebuilder.ImageIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x02id\x12\"\n\x0corganization\x18\x02 \x01(\tR\x0corganization\x12\x41\n\nproject_id\x18\x03 \x01(\x0b\x32\".cloudidl.common.ProjectIdentifierR\tprojectId\"F\n\x10GetImageResponse\x12\x32\n\x05image\x18\x01 \x01(\x0b\x32\x1c.cloudidl.imagebuilder.ImageR\x05imageB\xd1\x01\n\x19\x63om.cloudidl.imagebuilderB\x0cPayloadProtoH\x02P\x01Z/github.com/unionai/cloud/gen/pb-go/imagebuilder\xa2\x02\x03\x43IX\xaa\x02\x15\x43loudidl.Imagebuilder\xca\x02\x15\x43loudidl\\Imagebuilder\xe2\x02!Cloudidl\\Imagebuilder\\GPBMetadata\xea\x02\x16\x43loudidl::Imagebuilderb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'imagebuilder.payload_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\031com.cloudidl.imagebuilderB\014PayloadProtoH\002P\001Z/github.com/unionai/cloud/gen/pb-go/imagebuilder\242\002\003CIX\252\002\025Cloudidl.Imagebuilder\312\002\025Cloudidl\\Imagebuilder\342\002!Cloudidl\\Imagebuilder\\GPBMetadata\352\002\026Cloudidl::Imagebuilder'
  _GETIMAGEREQUEST.fields_by_name['id']._options = None
  _GETIMAGEREQUEST.fields_by_name['id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_GETIMAGEREQUEST']._serialized_start=135
  _globals['_GETIMAGEREQUEST']._serialized_end=321
  _globals['_GETIMAGERESPONSE']._serialized_start=323
  _globals['_GETIMAGERESPONSE']._serialized_end=393
# @@protoc_insertion_point(module_scope)



from flyteidl.event import cloudevents_pb2 as _cloudevents_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class EventType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    UNSPECIFIED: _ClassVar[EventType]
    WORKFLOW_EXECUTION: _ClassVar[EventType]
    NODE_EXECUTION: _ClassVar[EventType]
    TASK_EXECUTION: _ClassVar[EventType]
UNSPECIFIED: EventType
WORKFLOW_EXECUTION: EventType
NODE_EXECUTION: EventType
TASK_EXECUTION: EventType

class EventFilters(_message.Message):
    __slots__ = ["event_types"]
    EVENT_TYPES_FIELD_NUMBER: _ClassVar[int]
    event_types: _containers.RepeatedScalarFieldContainer[EventType]
    def __init__(self, event_types: _Optional[_Iterable[_Union[EventType, str]]] = ...) -> None: ...

class StreamEventsRequest(_message.Message):
    __slots__ = ["filters"]
    FILTERS_FIELD_NUMBER: _ClassVar[int]
    filters: EventFilters
    def __init__(self, filters: _Optional[_Union[EventFilters, _Mapping]] = ...) -> None: ...

class StreamEventsResponse(_message.Message):
    __slots__ = ["workflow_execution", "node_execution", "task_execution", "id"]
    WORKFLOW_EXECUTION_FIELD_NUMBER: _ClassVar[int]
    NODE_EXECUTION_FIELD_NUMBER: _ClassVar[int]
    TASK_EXECUTION_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    workflow_execution: _cloudevents_pb2.CloudEventWorkflowExecution
    node_execution: _cloudevents_pb2.CloudEventNodeExecution
    task_execution: _cloudevents_pb2.CloudEventTaskExecution
    id: str
    def __init__(self, workflow_execution: _Optional[_Union[_cloudevents_pb2.CloudEventWorkflowExecution, _Mapping]] = ..., node_execution: _Optional[_Union[_cloudevents_pb2.CloudEventNodeExecution, _Mapping]] = ..., task_execution: _Optional[_Union[_cloudevents_pb2.CloudEventTaskExecution, _Mapping]] = ..., id: _Optional[str] = ...) -> None: ...

class AcknowledgeEventRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class AcknowledgeEventResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...



from union.internal.app import app_definition_pb2 as _app_definition_pb2
from union.internal.common import identifier_pb2 as _identifier_pb2
from union.internal.common import list_pb2 as _list_pb2
from union.internal.validate.validate import validate_pb2 as _validate_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class CreateRequest(_message.Message):
    __slots__ = ["app"]
    APP_FIELD_NUMBER: _ClassVar[int]
    app: _app_definition_pb2.App
    def __init__(self, app: _Optional[_Union[_app_definition_pb2.App, _Mapping]] = ...) -> None: ...

class CreateResponse(_message.Message):
    __slots__ = ["app"]
    APP_FIELD_NUMBER: _ClassVar[int]
    app: _app_definition_pb2.App
    def __init__(self, app: _Optional[_Union[_app_definition_pb2.App, _Mapping]] = ...) -> None: ...

class GetRequest(_message.Message):
    __slots__ = ["app_id", "ingress"]
    APP_ID_FIELD_NUMBER: _ClassVar[int]
    INGRESS_FIELD_NUMBER: _ClassVar[int]
    app_id: _app_definition_pb2.Identifier
    ingress: _app_definition_pb2.Ingress
    def __init__(self, app_id: _Optional[_Union[_app_definition_pb2.Identifier, _Mapping]] = ..., ingress: _Optional[_Union[_app_definition_pb2.Ingress, _Mapping]] = ...) -> None: ...

class GetResponse(_message.Message):
    __slots__ = ["app"]
    APP_FIELD_NUMBER: _ClassVar[int]
    app: _app_definition_pb2.App
    def __init__(self, app: _Optional[_Union[_app_definition_pb2.App, _Mapping]] = ...) -> None: ...

class UpdateRequest(_message.Message):
    __slots__ = ["app", "reason"]
    APP_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    app: _app_definition_pb2.App
    reason: str
    def __init__(self, app: _Optional[_Union[_app_definition_pb2.App, _Mapping]] = ..., reason: _Optional[str] = ...) -> None: ...

class UpdateResponse(_message.Message):
    __slots__ = ["app"]
    APP_FIELD_NUMBER: _ClassVar[int]
    app: _app_definition_pb2.App
    def __init__(self, app: _Optional[_Union[_app_definition_pb2.App, _Mapping]] = ...) -> None: ...

class DeleteRequest(_message.Message):
    __slots__ = ["app_id"]
    APP_ID_FIELD_NUMBER: _ClassVar[int]
    app_id: _app_definition_pb2.Identifier
    def __init__(self, app_id: _Optional[_Union[_app_definition_pb2.Identifier, _Mapping]] = ...) -> None: ...

class DeleteResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class ListRequest(_message.Message):
    __slots__ = ["request", "org", "cluster_id", "project"]
    REQUEST_FIELD_NUMBER: _ClassVar[int]
    ORG_FIELD_NUMBER: _ClassVar[int]
    CLUSTER_ID_FIELD_NUMBER: _ClassVar[int]
    PROJECT_FIELD_NUMBER: _ClassVar[int]
    request: _list_pb2.ListRequest
    org: str
    cluster_id: _identifier_pb2.ClusterIdentifier
    project: _identifier_pb2.ProjectIdentifier
    def __init__(self, request: _Optional[_Union[_list_pb2.ListRequest, _Mapping]] = ..., org: _Optional[str] = ..., cluster_id: _Optional[_Union[_identifier_pb2.ClusterIdentifier, _Mapping]] = ..., project: _Optional[_Union[_identifier_pb2.ProjectIdentifier, _Mapping]] = ...) -> None: ...

class ListResponse(_message.Message):
    __slots__ = ["apps", "token"]
    APPS_FIELD_NUMBER: _ClassVar[int]
    TOKEN_FIELD_NUMBER: _ClassVar[int]
    apps: _containers.RepeatedCompositeFieldContainer[_app_definition_pb2.App]
    token: str
    def __init__(self, apps: _Optional[_Iterable[_Union[_app_definition_pb2.App, _Mapping]]] = ..., token: _Optional[str] = ...) -> None: ...

class WatchRequest(_message.Message):
    __slots__ = ["org", "cluster_id", "project", "app_id"]
    ORG_FIELD_NUMBER: _ClassVar[int]
    CLUSTER_ID_FIELD_NUMBER: _ClassVar[int]
    PROJECT_FIELD_NUMBER: _ClassVar[int]
    APP_ID_FIELD_NUMBER: _ClassVar[int]
    org: str
    cluster_id: _identifier_pb2.ClusterIdentifier
    project: _identifier_pb2.ProjectIdentifier
    app_id: _app_definition_pb2.Identifier
    def __init__(self, org: _Optional[str] = ..., cluster_id: _Optional[_Union[_identifier_pb2.ClusterIdentifier, _Mapping]] = ..., project: _Optional[_Union[_identifier_pb2.ProjectIdentifier, _Mapping]] = ..., app_id: _Optional[_Union[_app_definition_pb2.Identifier, _Mapping]] = ...) -> None: ...

class CreateEvent(_message.Message):
    __slots__ = ["app"]
    APP_FIELD_NUMBER: _ClassVar[int]
    app: _app_definition_pb2.App
    def __init__(self, app: _Optional[_Union[_app_definition_pb2.App, _Mapping]] = ...) -> None: ...

class UpdateEvent(_message.Message):
    __slots__ = ["updated_app", "old_app"]
    UPDATED_APP_FIELD_NUMBER: _ClassVar[int]
    OLD_APP_FIELD_NUMBER: _ClassVar[int]
    updated_app: _app_definition_pb2.App
    old_app: _app_definition_pb2.App
    def __init__(self, updated_app: _Optional[_Union[_app_definition_pb2.App, _Mapping]] = ..., old_app: _Optional[_Union[_app_definition_pb2.App, _Mapping]] = ...) -> None: ...

class DeleteEvent(_message.Message):
    __slots__ = ["app"]
    APP_FIELD_NUMBER: _ClassVar[int]
    app: _app_definition_pb2.App
    def __init__(self, app: _Optional[_Union[_app_definition_pb2.App, _Mapping]] = ...) -> None: ...

class WatchResponse(_message.Message):
    __slots__ = ["create_event", "update_event", "delete_event"]
    CREATE_EVENT_FIELD_NUMBER: _ClassVar[int]
    UPDATE_EVENT_FIELD_NUMBER: _ClassVar[int]
    DELETE_EVENT_FIELD_NUMBER: _ClassVar[int]
    create_event: CreateEvent
    update_event: UpdateEvent
    delete_event: DeleteEvent
    def __init__(self, create_event: _Optional[_Union[CreateEvent, _Mapping]] = ..., update_event: _Optional[_Union[UpdateEvent, _Mapping]] = ..., delete_event: _Optional[_Union[DeleteEvent, _Mapping]] = ...) -> None: ...

class UpdateStatusRequest(_message.Message):
    __slots__ = ["app"]
    APP_FIELD_NUMBER: _ClassVar[int]
    app: _app_definition_pb2.App
    def __init__(self, app: _Optional[_Union[_app_definition_pb2.App, _Mapping]] = ...) -> None: ...

class UpdateStatusResponse(_message.Message):
    __slots__ = ["app"]
    APP_FIELD_NUMBER: _ClassVar[int]
    app: _app_definition_pb2.App
    def __init__(self, app: _Optional[_Union[_app_definition_pb2.App, _Mapping]] = ...) -> None: ...

class LeaseRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: _identifier_pb2.ClusterIdentifier
    def __init__(self, id: _Optional[_Union[_identifier_pb2.ClusterIdentifier, _Mapping]] = ...) -> None: ...

class LeaseResponse(_message.Message):
    __slots__ = ["apps"]
    APPS_FIELD_NUMBER: _ClassVar[int]
    apps: _containers.RepeatedCompositeFieldContainer[_app_definition_pb2.App]
    def __init__(self, apps: _Optional[_Iterable[_Union[_app_definition_pb2.App, _Mapping]]] = ...) -> None: ...
